#include <gsl/gsl_sf.h>
#include <cmath>

#include "../math/mellin_Dk.hh"
#include "../parameters.hh"
#include "gammaLO.hh"


// LO anomalous dimensions gamma_0 
// defined by gamma = alpha_s gamma_0 + ...

dcomplex gamma0qq(dcomplex N) {
  return CF/2./M_PI * ( 1./N
			- 1./(N+1.)
			- 2.*psi0(N+1.) - 2.*EulerGamma
			+ 3./2. );
}
dcomplex gamma0qg(dcomplex N, double nf) {
  return nf/2./M_PI * ( 2.+N+N*N ) / N/(N+1.)/(N+2.);
}
dcomplex gamma0gq(dcomplex N) {
  return CF/2./M_PI * ( 2.+N+N*N ) / N/(N*N-1.);
}
dcomplex gamma0gg(dcomplex N, double nf) {
  return CA/M_PI * ( 11./12.
		     + 1./(N-1.)
		     - 1./N
		     + 1./(N+1.)
		     - 1./(N+2.)
		     - psi0(N+1.) - EulerGamma )
    - nf/6./M_PI;
}

// first derivatives
dcomplex gamma0qq_prime(dcomplex N) {
  return CF/2./M_PI * ( -1./N/N
			+ 1./(N+1.)/(N+1.)
			- 2.*psi_n(1,N+1.)
			);
}
dcomplex gamma0qg_prime(dcomplex N, double nf) {
  return nf/2./M_PI * ( - 1./N/N
			+ 2./(N+1.)/(N+1.)
			- 2./(N+2.)/(N+2.)
			);
}
dcomplex gamma0gq_prime(dcomplex N) {
  return CF/2./M_PI * ( - 2./(N-1.)/(N-1.)
			+ 2./N/N
			- 1./(N+1.)/(N+1.)
			);
}
dcomplex gamma0gg_prime(dcomplex N) {
  return CA/M_PI * ( -1./(N-1.)/(N-1.)
		     + 1./N/N
		     - 1./(N+1.)/(N+1.)
		     + 1./(N+2.)/(N+2.)
		     - psi_n(1,N+1.) );
}

// second derivatives
dcomplex gamma0qq_second(dcomplex N) {
  return CF/2./M_PI * ( 2./N/N/N
			- 2./(N+1.)/(N+1.)/(N+1.)
			- 2.*psi_n(2,N+1.)
			);
}
dcomplex gamma0qg_second(dcomplex N, double nf) {
  return nf/2./M_PI * ( 2./N/N/N
			- 4./(N+1.)/(N+1.)/(N+1.)
			+ 4./(N+2.)/(N+2.)/(N+2.)
			);
}
dcomplex gamma0gq_second(dcomplex N) {
  return CF/2./M_PI * ( 4./(N-1.)/(N-1.)/(N-1.)
			- 4./N/N/N
			+ 2./(N+1.)/(N+1.)/(N+1.)
			);
}
dcomplex gamma0gg_second(dcomplex N) {
  return CA/M_PI * ( 2./(N-1.)/(N-1.)/(N-1.)
		     - 2./N/N/N
		     + 2./(N+1.)/(N+1.)/(N+1.)
		     - 2./(N+2.)/(N+2.)/(N+2.)
		     - psi_n(2,N+1.) );
}







// real versions
////////////////
double gamma0qq(double N) {
  return CF/2./M_PI * ( 1./N
			- 1./(N+1.)
			- 2.*gsl_sf_psi(N+1.) - 2.*EulerGamma
			+ 3./2. );
}
double gamma0qg(double N, double nf) {
  return nf/2./M_PI * ( 2.+N+N*N ) / N/(N+1.)/(N+2.);
}
double gamma0gq(double N) {
  return CF/2./M_PI * ( 2.+N+N*N ) / N/(N*N-1.);
}
double gamma0gg(double N, double nf) {
  return CA/M_PI * ( 11./12.
		     + 1./(N-1.)
		     - 1./N
		     + 1./(N+1.)
		     - 1./(N+2.)
		     - gsl_sf_psi(N+1.) - EulerGamma )
    - nf/6./M_PI;
}

// derivatives
double gamma0qq_prime(double N) {
  return CF/2./M_PI * ( -1./N/N
			+ 1./(N+1.)/(N+1.)
			- 2.*gsl_sf_psi_1(N+1.)
			);
}
double gamma0qg_prime(double N, double nf) {
  return nf/2./M_PI * ( - 1./N/N
			+ 2./(N+1.)/(N+1.)
			- 2./(N+2.)/(N+2.)
			);
}
double gamma0gq_prime(double N) {
  return CF/2./M_PI * ( - 2./(N-1.)/(N-1.)
			+ 2./N/N
			- 1./(N+1.)/(N+1.)
			);
}
double gamma0gg_prime(double N) {
  return CA/M_PI * ( -1./(N-1.)/(N-1.)
		     + 1./N/N
		     - 1./(N+1.)/(N+1.)
		     + 1./(N+2.)/(N+2.)
		     - gsl_sf_psi_1(N+1.) );
}

