#include "NNLO_finite-mt/nlo_finite-mt.hh"
dcomplex L(dcomplex N) {
  return -2.*( EulerGamma + psi0(N+1.) )/N +1./N/N;
}
dcomplex NFO_1_gg(dcomplex N, double mu) {
  dcomplex res = 0;
  if(finite_mt) {
    if(mu<2*mtop) {
      double c1 = sxc->c1marzani(mu,mtop);
      res = gg_nlo_mt_N(N, mu, mtop, 1.) + smallx_match_nlo_N(N, c1);
    } else {
      res = gg_nlo_infmt_N(N, mu, mtop, 1.);
    }
  } else {  // pointlike
    res = 4.*CA*mellD(1,N)
      + 2*CA * ZETA2 + 11./2.
      + CA * 2.*( L(N-1) - 2.*L(N) + L(N+1) - L(N+2) + psi_n(1,N) )
      - 11./2.*( 1/(N-1.) -3./N +3./(N+1.) -1./(N+2.) );
  }
  return res;
}
dcomplex NFO_1_qg(dcomplex N, double mu) {
  dcomplex res = 0;
  if(finite_mt) {
    if(mu<2*mtop) {
      double c1 = sxc->c1marzani(mu,mtop);
      double c10 = sxc->c10(mu,mtop);
      res = qg_nlo_mt_N(N, mu, mtop, 1.) + smallx_match_nlo_N(N, (c1-c10)*CF/CA);
    } else {
      res = qg_nlo_infmt_N(N, mu, mtop, 1.);
    }
  } else {  // pointlike
    res = 2./3.*(2*L(N-1.)-2*L(N)+L(N+1.)) -1./(N-1.) +2./N -1./3./(N+1.);
  }
  return res;
}
dcomplex NFO_1_qqb(dcomplex N, double mu) {
  dcomplex res = 0;
  if(finite_mt) {
    if(mu<2*mtop) {
      double c1 = sxc->c1marzani(mu,mtop);
      double c10 = sxc->c10(mu,mtop);
      res = qqb_nlo_mt_N(N, mu, mtop, 1.) + smallx_match_nlo_N(N, (c1-2*c10)*pow(CF/CA,2));
    } else {
      res = qqb_nlo_infmt_N(N, mu, mtop, 1.);
    }
  } else {  // pointlike
    res = 32./27.*(1./(N-1.) - 3./N + 3./(N+1.) -1./(N+2.));
  }
  return res;
}




dcomplex NFO_2_gg(dcomplex N, double mu, double muFmuratio=1) {
  dcomplex res;
  if(mu<2*mtop && finite_mt) {
    double lf = -2.*log(muFmuratio);
    double c1 = sxc->c1marzani(mu,mtop);
    double c2 = sxc->c2marzani(mu,mtop);
    c2  += (c1+lf)*lf*2.;
    res = gg_nnlo_mt_N(N-1., mu, mtop, muFmuratio) + smallx_match(N-1, c2);
  } else
    res = gg_nnlo_infmt_N(N-1., mu, mtop, muFmuratio);
  return res;
}
dcomplex NFO_2_qg(dcomplex N, double mu, double muFmuratio=1) {
  dcomplex res;
  if(mu<2*mtop && finite_mt) {
    double lf = -2.*log(muFmuratio);
    double c10 = sxc->c10(mu,mtop);
    double c20 = sxc->c20(mu,mtop);
    double c11 = sxc->c11(mu,mtop);
    double c1 = 2*c10;
    double c2 = 2.*c20+c11;
    c2  += (c1+lf)*lf*2.;
    c20 += (c1+lf)*lf/2.;
    res = qg_nnlo_mt_N(N-1, mu, mtop, muFmuratio) + smallx_match(N-1, (c2-c20)*CF/CA);
  } else
    res = qg_nnlo_mt_N(N-1, mu, mtop, muFmuratio);
  //res = qg_nnlo_infmt_N(N-1, mu, mtop, muFmuratio);
  return res;
}
dcomplex NFO_2_qqb(dcomplex N, double mu, double muFmuratio=1) {
  dcomplex res;
  if(mu<2*mtop && finite_mt) {
    double lf = -2.*log(muFmuratio);
    double c10 = sxc->c10(mu,mtop);
    double c20 = sxc->c20(mu,mtop);
    double c11 = sxc->c11(mu,mtop);
    double c1 = 2*c10;
    double c2 = 2.*c20+c11;
    c2  += (c1+lf)*lf*2.;
    c20 += (c1+lf)*lf/2.;
    res = qu_nnlo_mt_N(N-1, mu, mtop, muFmuratio) + smallx_match(N-1, (c2-2*c20)*pow(CF/CA,2));
  } else
    res = qu_nnlo_mt_N(N-1, mu, mtop, muFmuratio);
  //res = qu_nnlo_infmt_N(N-1, mu, mtop, muFmuratio);
  return res;
}
dcomplex NFO_2_qq(dcomplex N, double mu, double muFmuratio=1) {
  dcomplex res;
  if(mu<2*mtop && finite_mt) {
    double lf = -2.*log(muFmuratio);
    double c10 = sxc->c10(mu,mtop);
    double c20 = sxc->c20(mu,mtop);
    double c11 = sxc->c11(mu,mtop);
    double c1 = 2*c10;
    double c2 = 2.*c20+c11;
    c2  += (c1+lf)*lf*2.;
    c20 += (c1+lf)*lf/2.;
    res = qq_nnlo_mt_N(N-1, mu, mtop, muFmuratio) + smallx_match(N-1, (c2-2*c20)*pow(CF/CA,2));
  } else
    res = qq_nnlo_mt_N(N-1, mu, mtop, muFmuratio);
  //res = qq_nnlo_infmt_N(N-1, mu, mtop, muFmuratio);
  return res;
}
dcomplex NFO_2_qqp(dcomplex N, double mu, double muFmuratio=1) {
  dcomplex res;
  if(mu<2*mtop && finite_mt) {
    double lf = -2.*log(muFmuratio);
    double c10 = sxc->c10(mu,mtop);
    double c20 = sxc->c20(mu,mtop);
    double c11 = sxc->c11(mu,mtop);
    double c1 = 2*c10;
    double c2 = 2.*c20+c11;
    c2  += (c1+lf)*lf*2.;
    c20 += (c1+lf)*lf/2.;
    res = qqp_nnlo_mt_N(N-1, mu, mtop, muFmuratio) + smallx_match(N-1, (c2-2*c20)*pow(CF/CA,2));
  } else
    res = qqp_nnlo_mt_N(N-1, mu, mtop, muFmuratio);
  //res = qqp_nnlo_infmt_N(N-1, mu, mtop, muFmuratio);
  return res;
}


