#include <iomanip>
#include <vector>
#include <algorithm>
#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
#include <stdio.h>
#include <cmath>
#include <ctime>
#include <cstdlib>
#include <sys/stat.h> 
#include <sys/time.h> 
#include <gsl/gsl_sf.h> 
#include <gsl/gsl_interp.h>
#include <gsl/gsl_spline.h>
#include <getopt.h> 

#include "LHAPDF/LHAPDF.h"  // interface with LHAPDF

#include "math/complex.hh"
#include "math/mellin_Dk.hh"
#include "chebyshev.hh"
#include "NLO_finite-mt/bdv.hh"
#include "NNLO_pointlike/NNLOpl.hh"
#include "NNLO_finite-mt/nnlo_finite-mt.hh"
#include "ggHiggs.hh"
#include "parameters.hh"
#include "anomalous_dimensions/gammaLO.hh"
#include "anomalous_dimensions/gammaNLO.hh"
#include "anomalous_dimensions/gammaNNLO.hh"

#include <cuba.h>


using namespace std;



const int _nf = 5;


// PRIVATE flags and variables ( don't change here !!! )
double mtop = 172.5;
double mbot = 4.4; // negative mass means no bottom running in the loop
double mcha = 1.2; // negative mass means no charm running in the loop
int    order = 1;
//int PDForder = 1;
bool finite_mt = false;
SmallxCoeffs *sxc;
double _muF_mu_ratio  = 1.;
double _muR_muF_ratio = 1.;
double muF, muR, muM;
bool __exact__ = false;
bool __soft_part__ = true;
bool __hard_part__ = true;
int __log_type__ = 0; // 0:D, 1:Dhat, 2:Dlog
bool __Nsoftcoll__ = false;
bool __NNLL__ = false;
int APtype = 0;
bool APimprExponent = true;
bool MSbar = true;
bool quiet = true;
double INTEGRAL_PRECISION = 15.e-3;
bool exactmuRdep = true;
bool exactmuFdep = false;
bool muRdepggonly = true;
bool muFdepggonly = true;







double Li2(double x){
 double x_0 = -0.30;
 double x_1 = 0.25;
 double x_2 = 0.51;
 if (x == 1.) return ZETA2;
 if (x <= x_0){ 
   double temp = log(ABS(1.0-x));
   return -Li2(-x/(1.0-x)) - temp*temp/2 ; }
 else if (x < x_1){
   double z = - log(1.0-x);
   double temp = z*(1.0-z/4.0*(1.0-z/9.0*(1.0-z*z/100.0
                  *(1.0-5.0*z*z/294.0*(1.0-7.0*z*z/360.0
                  *(1.0-5.0*z*z/242.0*(1.0-7601.0*z*z/354900.0
                  *(1.0-91.0*z*z/4146.0*(1.0-3617.0*z*z/161840.0)
                   ))))))));
   return temp; }
   else if (x < x_2) return - Li2(-x) + Li2(x*x)/2.0 ;
   else { return ZETA2 - Li2(1.0-x) 
                  - log(ABS(x))*log(ABS(1.0-x)) ; }
}
double Li3(double x){
 double x_0 = -1.0;
 double x_1 = -0.85;
 double x_2 = 0.25;
 double x_3 = 0.63;
 double x_4 =  1.0;
 if (x == 1.) return ZETA3;
 if (x == -1.) return - 0.75 * ZETA3;
 if (x <= x_0){ 
   double lnx = log(-x);
   return Li3(1.0/x) - ZETA2*lnx - lnx*lnx*lnx/6.0; }
 else if (x < x_1){
   return Li3(x*x)/4.0 - Li3(-x); }
   else if (x < x_2){
     double z = - log(1.0-x);
     double temp = z*(1.0-3.0*z/8.0*(1.0-17.0*z/81.0*(1.0-15*z/136.0
                    *(1.0-28.0*z/1875.0*(1.0+5.0*z/8.0*(1.0-304.0*z/7203.0
                    *(1.0+945.0*z/2432.0*(1.0-44.0*z/675.0*(1.0+7.0*z/24.0
                    *(1.0-26104.0*z/307461.0*(1.0+1925.0*z/8023.0
                    *(1.0-53598548.0*z/524808375.0
                    *(1.0+22232925.0*z/107197096.0
                     )))))))))))));
     return temp; }
     else if (x < x_3){
       return Li3(x*x)/4.0 - Li3(-x); }
       else if (x < x_4){
         double ln1x = log(1.0-x); 
         return -Li3(1.0-x) - Li3(-x/(1.0-x)) + ZETA3 + ZETA2*ln1x
	   - log(x)*ln1x*ln1x/2.0 + ln1x*ln1x*ln1x/6.0; }
       else { 
         double lnx = log(x);
         return Li3(1./x) + 2.0*ZETA2*lnx - lnx*lnx*lnx/6.0; }
}






// 1-d integration
#include<gsl/gsl_integration.h>
double gauss(double f(double,void*), double xmin, double xmax, double prec, double *error, void *par) {
  double result, err;
  gsl_function F;
  gsl_integration_workspace * w = gsl_integration_workspace_alloc (1000);
  F.function = f;
  F.params=par;
  gsl_integration_qags (&F, xmin, xmax, 0, prec, 1000,    w, &result, &err);
  gsl_integration_workspace_free (w);
  if(error) *error = err;
  return result;
}


// alpha_s running from ihixs
double alphas_ihixs(double mu, int porder, double as0, double mu0=mZ) {
  // Nf = 5 **************** <---------
  double bb0 = 1.916666667;
  double bb1 = 1.260869565;
  double bb2 = 1.474788647;
  double bb3 = 9.835916431;
  //
  double as;
  as0 /= M_PI;
  //double as0 = asZ/M_PI;
  //double as0=asZ/3.1415926;
  //double Mz=91.187;
  //
  int N = 100000;
  //double step = 2.*log(mu/mZ)/N, incr;
  //double step = 2.*log(mu/Mz)/N, incr;
  double step = 2.*log(mu/mu0)/N, incr;
  for (int i=0; i<N; i++) {
    if (porder >= 3)
      incr = step*bb0*as0*as0*(1+bb1*as0 + bb2*as0*as0 + bb3*as0*as0*as0 );
    else if (porder == 2)
      incr = step*bb0*as0*as0*(1+bb1*as0 + bb2*as0*as0 );
    else if (porder == 1)
      incr = step*bb0*as0*as0*(1+bb1*as0);
    else if (porder == 0)
      incr = step*bb0*as0*as0;
    as = as0 - incr;
    as0 = as;
  }
  return as*M_PI;
}







double b0 = (33.-2.*_nf)/12.;
double b1 = (153.-19.*_nf)/24.;
double b2 = (2857. - 5033./9.*_nf + 325./27.*_nf*_nf)/128.;


dcomplex InverseMellinIntegrand(dcomplex N, double log1ox, dcomplex f(dcomplex, void*), void *par) {
  return exp( N*log1ox ) * f(N,par) / M_PI;
}
double InverseMellin(double z, dcomplex f(dcomplex, void*), void *par) { // Talbot algorithm
  if(z==1.) return 0;
  double log1ox = -log(z);
  double res = 0.; 
  // prec ~ 10^{-0.6*M}
  int M = 13;
  double r = 1.+2.*M/5./log1ox;
  double t, cott, sigma;
  dcomplex N;
  res = real(0.5 * InverseMellinIntegrand(r, log1ox, f, par));
  for(int k=1; k<M; k++) {
    t = k*M_PI/M;
    cott = 1./tan(t);
    sigma = t + cott*(t*cott - 1.);
    N = r*t*(cott+I);
    res += real(InverseMellinIntegrand(N, log1ox, f, par) * (1.+I*sigma));
  }
  res *= M_PI*r/M;
  return res;
}




dcomplex gamma0poles(dcomplex N, int nf) {
  double e01 = CA;
  double e00 = (-11.*CA+2.*nf*(2.*CF/CA-1.))/12.;
  return (e01/(N-1) + e00);
}
dcomplex gamma1poles(dcomplex N, int nf) {
  double e11 = nf*(26.*CF-23.*CA)/36.;
  return e11/(N-1);
}
dcomplex gamma2poles(dcomplex N, int nf) {
  double e22 = CA*( (18.*ZETA2-71.)*(CA-2*CF)*nf + CA*CA*(54*ZETA3+99.*ZETA2-395) ) /108.;
  return e22/(N-1)/(N-1);
}
dcomplex FO_1_sx_NLL_RC(dcomplex N, double mu) {
  double lf = -2.*log(_muF_mu_ratio);
  double c10 = sxc->c10(mu,mtop);
  double c1 = 2*c10;
  c1 += 2.*lf;
  return c1*gamma0poles(N,_nf);
}
dcomplex FO_2_sx_NLL_RC(dcomplex N, double mu) {
  double lf = -2.*log(_muF_mu_ratio);
  double c10 = sxc->c10(mu,mtop);
  double c20 = sxc->c20(mu,mtop);
  double c11 = sxc->c11(mu,mtop);
  double c1 = 2*c10;
  double c2 = 2.*c20+c11;
  c20 += (c1+lf)*lf/2.;
  c2  += (c1+lf)*lf*2.;
  c1  += 2*lf;
  dcomplex g0 = gamma0poles(N,_nf);
  dcomplex g1 = gamma1poles(N,_nf);
  return c2*g0*g0 - 2*c20*b0*g0 + c1*g1 ;
}
dcomplex FO_3_sx_NLL_RC(dcomplex N, double mu) {
  double lf = -2.*log(_muF_mu_ratio);
  double c10 = sxc->c10(mu,mtop);
  double c20 = sxc->c20(mu,mtop);
  double c11 = sxc->c11(mu,mtop);
  double c30 = sxc->c30(mu,mtop);
  double c21 = sxc->c21(mu,mtop);
  if(MSbar) c30 += 8./3.*ZETA3;
  double c1 = 2*c10;
  double c2 = 2.*c20+c11;
  double c3 = 2.*(c30+c21);
  // scale dependence
  c30 += c20*lf + c1*lf*lf/4. + lf*lf*lf/6.;
  c21 += (c2-c20)*lf + 3.*c1*lf*lf/4. + lf*lf*lf/2.;
  c20 += (c1+lf)*lf/2.;
  c2  += (c1+lf)*lf*2.;
  c1  += 2*lf;
  dcomplex g0 = gamma0poles(N,_nf);
  dcomplex g1 = gamma1poles(N,_nf);
  dcomplex g2 = gamma2poles(N,_nf);
  dcomplex hard = (c3*g0*g0*g0 - (3*c30+c21)*2*b0*g0*g0 + 4*c30*b0*b0*g0
		  + 2*c2*g0*g1 - 4*c20*b0*g1 + c1*g2 );
  //double lrf = 2.*log(_muR_muF_ratio);
  //hard += 4*b0*FO_2_sx_NLL_RC(N,mu)*lrf;
  return hard;
}
double MomConsCoeff[3] = {0,0,0};
void ComputeMomConsCoeff(double mu) {
  MomConsCoeff[0] = real(FO_1_sx_NLL_RC(2,mu) - 2.*FO_1_sx_NLL_RC(3,mu) + FO_1_sx_NLL_RC(4,mu));
  MomConsCoeff[1] = real(FO_2_sx_NLL_RC(2,mu) - 2.*FO_2_sx_NLL_RC(3,mu) + FO_2_sx_NLL_RC(4,mu));
  MomConsCoeff[2] = real(FO_3_sx_NLL_RC(2,mu) - 2.*FO_3_sx_NLL_RC(3,mu) + FO_3_sx_NLL_RC(4,mu));
}
void ResetMomConsCoeff() {
  MomConsCoeff[0] = 0;  MomConsCoeff[1] = 0;  MomConsCoeff[2] = 0;
}




double Dk(int k, double z) {
  return pow(log(1-z),k)/(1.-z);
}
double Dtk(int k, double z) {
  return pow(log(1-z)-0.5*log(z),k)/(1.-z);
}
double DMPk(int k, double z) {
  return pow(log(log(1./z)),k)/log(1./z);
}
double mell_const(int k, double fake=0) {
  return DG(k+1,1.)/(k+1.);
}




// perturbative expansions of constants and logs
double _cd1, _cd2, _cd3;
void init_delta_const() {
  _cd1 = 2.*CA * ZETA2;
  _cd2 = ( 837./16. + 67./2.*ZETA2 - 9./20.*ZETA2*ZETA2 - 165./4.*ZETA3
	   + _nf  * ( 5./6.*ZETA3 - 5./3.*ZETA2 - 247./36. ) );
  _cd3 = 1740.02 - 131.928*_nf + 1.75719*_nf*_nf; // new coefficient
}
double _W1, _W2, _W3;
double _W1t, _W2t, _W3t;
void init_Wilson(double mu) {
  double lHt = 2.*log(mu/mtop);
  _W1 = 11./2.;
  _W2 = 121./16. + 2777./144. + 19./8.*lHt + (-67./48. + 2./3.*lHt)*_nf;
  //_W3 = 40.6488 + 40.1458*lHt + 10.941*lHt*lHt; // write full expression .... done.
  _W3 = (-1792967./20736. +  897943./4608.*ZETA3 + 5347./288.*lHt + 209./32.*lHt*lHt
	 + _nf * (493./10368. + 209./54.*lHt + 23./16.*lHt*lHt - 110779./6912.*ZETA3)
	 + _nf*_nf * (-6865./15552. + 77./864.*lHt - lHt*lHt/9. ) );
  //cout << _W1/pow(M_PI,1) << endl;
  //cout << _W2/pow(M_PI,2) << endl;
  //cout << _W3/pow(M_PI,3) << endl;
}
void init_Wilson_mt(double mu) {
  _W1t = G2l();
  _W2t = gg_nnlo_mt_delta(mu, mtop, 1) - _cd2 - _cd1*_W1t;
  _W3t = _W3; // pointlike value
  if(mu > 2*mtop) _W2t = _W2; // pointlike value
}
const double A1 = CA*CA*(67./36.-ZETA2/2.) - 5./18.*CA*_nf;
const double A2 = CA*CA*CA*(245./24.-67./9.*ZETA2+11./6.*ZETA3+11./5.*ZETA2*ZETA2)/4.
  + CA*CA*_nf*(-209./108.+10./9.*ZETA2-7./3.*ZETA3)/4.
  + CA*CF*_nf*(-55./24.+2.*ZETA3)/4.
  - CA*_nf*_nf/4./27.;
const double B1 = CA*CA*(2./3.+3./4.*ZETA3) - (CA/6.+CF/8.)*_nf;
const double B2 = CA*CA*CA/4.*(79./32.-ZETA2*ZETA3+ZETA2/6.+11./24.*ZETA2*ZETA2+67./6.*ZETA3-5.*ZETA5)
  - CA*CA*_nf/4.*(233./288.+ZETA2/6.+ZETA2*ZETA2/12.+5./3.*ZETA3)
  - CA*CF*_nf/4.*241./288.
  + CF*CF*_nf/4./16.
  + _nf*_nf/4.*(CA*29./288.+CF*11./144.);
//
// resummation coeffs
const double rA1 = CA/M_PI;
const double rA2 = CA/2.* (67.*CA/18.-5./9.*_nf-CA*ZETA2) /M_PI/M_PI;
const double rA3 = ( CA*CA* (245./24.-67.*ZETA2/9.+11.*ZETA3/6.+11.*ZETA2*ZETA2/5.) +
		     ( (2.*ZETA3 - 55./24.)*CF + (10.*ZETA2/9.-7.*ZETA3/3.-209./108.)*CA )*_nf
		     - _nf*_nf*1./27. ) *CA/4. /M_PI/M_PI/M_PI;
const double rD2 = CA * ( CA * (176.*ZETA2/3. + 56.*ZETA3 - 1616./27.) + _nf*(224./27.-32.*ZETA2/3.) )/16./M_PI/M_PI; // checked: OK.
const double rD3 = CA * ( CA*CA *(-594058./729. +98224./81.*ZETA2 +40144./27.*ZETA3 -2992./15.*ZETA2*ZETA2 -352./3.*ZETA2*ZETA3 -384.*ZETA5 )
			  +CA*_nf*( 125252./729. -29392./81.*ZETA2 -2480./9.*ZETA3 +736./15.*ZETA2*ZETA2 )
			  +CF*_nf*( 3422./27. - 32.*ZETA2 - 608./9.*ZETA3 - 64./5.*ZETA2*ZETA2 )
			  +_nf*_nf*( 320./27.*ZETA3 + 640./27.*ZETA2 - 3712./729.)) /64./M_PI/M_PI/M_PI; // checked: OK.
//
double g0bar1=0, g0bar2=0, g0bar3=0;
void initResConsts(double M) {
  //
  double d1 = _cd1 + _W1;
  double d2 = _cd2 + _cd1*_W1 + _W2;
  double d3 = _cd3 + _cd2*_W1 + _cd1*_W2 + _W3;
  if(finite_mt) {
    d1 = G2l() + 2.*CA * ZETA2;
    if(M < 2*mtop) d2 = gg_nnlo_mt_delta(M, mtop, 1);
    //d3 = (52.1348 - 1.97392*d1/M_PI) *M_PI*M_PI*M_PI;  // this corresponds to g0bar3 = 0 only for muF=mH
    //d3 = ; // not known -> use EFT value
  }
  double gA1 = CA*CA*(67./36.-ZETA2/2.) - 5./18.*CA*_nf;
  double gB1 = CA*CA*(2./3.+3./4.*ZETA3) - (CA/6.+CF/8.)*_nf;
  double gB2 = CA*CA*CA/4.*(79./32.-ZETA2*ZETA3+ZETA2/6.+11./24.*ZETA2*ZETA2+67./6.*ZETA3-5.*ZETA5)
    - CA*CA*_nf/4.*(233./288.+ZETA2/6.+ZETA2*ZETA2/12.+5./3.*ZETA3)
    - CA*CF*_nf/4.*241./288.
    + CF*CF*_nf/4./16.
    + _nf*_nf/4.*(CA*29./288.+CF*11./144.);
  //
  double lf  = 2.*log(M/muF);
  double lrf = 2.*log(muR/muF);
  double lr = 2.*log(muR/M);
  //
  //double a11 = 4*CA;
  double a23 = 8*CA*CA;
  double a22 = -4*CA*b0;
  double a21 = 4*CA*d1 + CA*CA*(67./9.-18.*ZETA2) - 10./9.*CA*_nf;
  double a20 = CA*CA*(-101./27.+11./3.*ZETA2+39./2.*ZETA3) + CA*_nf*(14./27.-2./3.*ZETA2);
  // factorization scale dependence (as=as(M))
  d3 += (4*b0*b0*b0 - 6*CA*CA*b0*ZETA2 + 8./3.*CA*CA*CA*ZETA3)*lf*lf*lf
    //
    + (b0*(6*gB1+b1) - 4*CA*gA1*ZETA2 + 20*CA*CA*b0*ZETA3 + 8*CA*CA*CA*(ZETA2*ZETA2-3*ZETA4) + d1*(3*b0*b0-2*CA*CA*ZETA2))*lf*lf
    //
    + (2*gB2+8*CA*gA1*ZETA3+2*gB1*d1+2*b0*d2+2*CA*(6*ZETA5*a23-2*ZETA4*a22+ZETA3*a21-ZETA2*a20))*lf;
  d2 += (-2.*CA*CA*ZETA2 + 3*b0*b0)*lf*lf + (2*gB1+2*b0*d1+8*CA*CA*ZETA3)*lf;
  d1 += 2*b0*lf;
  //
  // adding renormalization scale dependence
  d3 += (2*b2+3*b1*d1+4*b0*d2)*lr + (7*b0*b1+6*b0*b0*d1)*lr*lr + 4*pow(b0*lr,3);
  d2 += (2*b1+3*b0*d1)*lr + 3*b0*b0*lr*lr;
  d1 += 2*b0*lr;
  //
  // restore normalization
  d1 /= M_PI;
  d2 /= M_PI*M_PI;
  d3 /= M_PI*M_PI*M_PI;
  //
  // construct full g0k
  //const double c1 = EulerGamma;
  double bb0 = b0/M_PI;
  double bb1 = b1/b0/M_PI;
  double c2 = pow(EulerGamma,2) -ZETA2;
  double c3 = pow(EulerGamma,3) -3*EulerGamma*ZETA2 +2*ZETA3;
  double c4 = pow(EulerGamma,4) -6*EulerGamma*EulerGamma*ZETA2 +8*EulerGamma*ZETA3 +3*ZETA2*ZETA2 -6*ZETA4;
  double c5 = 24*ZETA5 +3*EulerGamma*ZETA2*ZETA2 +pow(EulerGamma,5) - 10*pow(EulerGamma,3)*ZETA2 +20*EulerGamma*EulerGamma*ZETA3 -20*ZETA3*ZETA2;
  double c6 = 144*ZETA5*EulerGamma +pow(EulerGamma,6) -15*pow(EulerGamma,4)*ZETA2 +40*pow(EulerGamma,3)*ZETA3 +9*EulerGamma*EulerGamma*ZETA2*ZETA2 -120*EulerGamma*ZETA3*ZETA2 +40*ZETA3*ZETA3 -45./7.*pow(ZETA2,3);
  //
  double S1 = 2*rA1* (ZETA2+EulerGamma*EulerGamma - EulerGamma*lf);
  double S2 = 4*rA1*bb0/3.*c3 -2*( rA1*bb0*(2*EulerGamma+lr) + rA2 )*c2
    - EulerGamma* ( rD2 - 4.*rA2*EulerGamma - 4.*rA1*bb0*(EulerGamma*EulerGamma+ZETA2)
		    - 4.*rA1*bb0*EulerGamma*lr +2*rA2*lf
		    - rA1*bb0*(lr*lr - lrf*lrf) );
  double S3 = -c4* (4*rA1*pow(bb0,2))/3.
    +c3* (4*bb0*(2*rA2 + rA1*(bb1 + 4*bb0*EulerGamma) - 4*rA1*bb0*log(M/muR)))/3.
    -c2* 2*(rA3 + bb0*(-rD2 + 4*rA2*EulerGamma + 
		     2*rA1*(bb1*EulerGamma + 2*bb0*(pow(EulerGamma,2) + ZETA2))) - 
	    2*bb0*(2*rA2 + rA1*(bb1 + 4*bb0*EulerGamma))*log(M/muR) + 
	    4*rA1*pow(bb0,2)*pow(log(M/muR),2))
    -EulerGamma* (3*rD3 - 12*rA3*EulerGamma + 12*bb0*rD2*EulerGamma - 24*rA2*bb0*pow(EulerGamma,2) - 
		  12*rA1*bb0*bb1*pow(EulerGamma,2) - 16*rA1*pow(bb0,2)*pow(EulerGamma,3) - 
		  24*rA2*bb0*ZETA2 - 12*rA1*bb0*bb1*ZETA2 - 48*rA1*pow(bb0,2)*EulerGamma*ZETA2 - 
		  32*rA1*pow(bb0,2)*ZETA3 + 12*
		  (rA3 + bb0*(-rD2 + 4*rA2*EulerGamma + 
			    2*rA1*(bb1*EulerGamma + 2*bb0*(pow(EulerGamma,2) + ZETA2))))*log(M/muR) - 
		  12*bb0*(2*rA2 + rA1*(bb1 + 4*bb0*EulerGamma))*pow(log(M/muR),2) + 
		  16*rA1*pow(bb0,2)*pow(log(M/muR),3) - 12*rA3*log(muF/muR) + 
		  12*bb0*(2*rA2 + rA1*bb1)*pow(log(muF/muR),2) - 
		  16*rA1*pow(bb0,2)*pow(log(muF/muR),3))/3.;
  // S1^2
  const double S1s = -4*rA1*rA1*(c4 -2*(2*EulerGamma-lf)*c3 +pow(2*EulerGamma-lf,2)*c2);
  // S1^3
  const double S1c = -8*rA1*rA1*rA1*(c6 -3*(2*EulerGamma-lf)*c5 +3*pow(2*EulerGamma-lf,2)*c4 -pow(2*EulerGamma-lf,3)*c3);
  // S1*S2
  const double S1S2 = 2*rA1*(4*rA1*bb0/3.*c5 -2*( rA1*bb0*(2*EulerGamma+lr) + rA2 )*c4
			    - c3* ( rD2 - 4.*rA2*EulerGamma - 4.*rA1*bb0*(EulerGamma*EulerGamma+ZETA2)
				    - 4.*rA1*bb0*EulerGamma*lr +2*rA2*lf
				    - rA1*bb0*(lr*lr - lrf*lrf) ))
    -2*rA1*(2*EulerGamma-lf)* (4*rA1*bb0/3.*c4 -2*( rA1*bb0*(2*EulerGamma+lr) + rA2 )*c3
			      - c2* ( rD2 - 4.*rA2*EulerGamma - 4.*rA1*bb0*(EulerGamma*EulerGamma+ZETA2)
				      - 4.*rA1*bb0*EulerGamma*lr +2*rA2*lf
				      - rA1*bb0*(lr*lr - lrf*lrf) ));
  //
  const double gD1 = S1;
  const double gD2 = S1s/2. + S2 + gD1*gD1;
  const double gD3 = S1c/6. + S1S2 + S3 + 2*gD1*gD2 - gD1*gD1*gD1;
  //
  double g01 = d1 + gD1;
  double g02 = d2 + d1*gD1 + gD2;
  double g03 = d3 + d2*gD1 + d1*gD2 + gD3; 
  //
  g0bar1 = d1;
  g0bar2 = g02 - g01*S1 - S2 + S1*S1/2.;
  g0bar3 = g03 - g02*S1 - g01*(S2-S1*S1/2.) - (S1*S1*S1/6. - S1*S2 + S3);
  //
  return;
}
//
void init_consts(double mH) {
  initBDV(mH, mtop, mbot, mcha);
  init_delta_const();
  init_Wilson(mH);
  init_Wilson_mt(mH);
}
void SetScales(double mH) {
  muM = mH;
  if(exactmuFdep) muF = mH;
  else muF = _muF_mu_ratio * mH;
  if(exactmuRdep) muR = muF;
  else muR = _muR_muF_ratio * muF;
  initResConsts(mH);
}
void SetScaleDependence(bool exact_muR, bool exact_muF, bool muR_gg_only, bool muF_gg_only) {
  exactmuRdep = exact_muR;
  exactmuFdep = exact_muF;
  muRdepggonly = muR_gg_only;
  muFdepggonly = muF_gg_only;
}




// Pointlike log contributions
double FO_1_log_PL(double z, double D(int,double)) {
  return 4.*CA*D(1,z);
}
double FO_2_log_PL(double z, double D(int,double)) {
  double value;
  value = ( 72. * D(3,z)
	    + (2.*_nf - 33.) * D(2,z)
	    + (67. - 90.*ZETA2 - 10./3.*_nf) * D(1,z)
	    + (-101./3. + 33.*ZETA2 + 351./2.*ZETA3 + _nf*(14./9.-2.*ZETA2)) * D(0,z)
	    );
  return value;
}
double FO_3_log_PL(double z, double D(int,double)) {
  double CA3 = CA*CA*CA;
  double value;
  value = ( 8*CA3 * D(5,z)
	    + (-110./9.*CA3 + 20/9.*CA*CA*_nf) * D(4,z)
	    + (CA3*(925./27. - 56*ZETA2) - 164./27.*CA*CA*_nf + 4./27.*CA*_nf*_nf) * D(3,z)
	    + (CA3 * (-1051./27. + 187./3*ZETA2 + 181*ZETA3)
	       + CA*CA*_nf * (457./2./27. - 34./3.*ZETA2)
	       + 0.5*CA*CF*_nf - 10./27.*CA*_nf*_nf ) * D(2,z)
	    + (CA3 * (30569./8./81. - 152./3.*ZETA2 - 352./3.*ZETA3 - 154./5.*ZETA2*ZETA2)
	       + CA*CA*_nf * (-4211./4./81. + 94./9.*ZETA2 + 46./3.*ZETA3)
	       - CA*CF*_nf * (63./8.-6.*ZETA3)
	       + CA*_nf*_nf * (25./81.-4./9.*ZETA2) ) * D(1,z)
	    + (CA3 * (-297029./32./729. + 8563./4./81.*ZETA2 + 8941./4./27.*ZETA3 + 253./4./15.*ZETA2*ZETA2
		      - 725./2./3.*ZETA2*ZETA3 + 186.*ZETA5)
	       + CA*CA*_nf * (31313./16./729. - 2173./4./81.*ZETA2 - 475./4./9.*ZETA3 - 17./2./15*ZETA2*ZETA2)
	       + CA*CF*_nf * (1711./32./27. - 0.5*ZETA2 - 19./2./9.*ZETA3 - 1./5.*ZETA2*ZETA2)
	       - CA*_nf*_nf * (58./729. - 10./27.*ZETA2 - 5./27.*ZETA3) ) * D(0,z)
	    );
  return value;
}
//
dcomplex mellD(int k, dcomplex N) {
  if(__log_type__ == 0)
    return mell_D(k,N);
  else if(__log_type__ == 1)
    return mell_DB(k,N);
  else if(__log_type__ == 2)
    return mell_DM(k,N);
  else if(__log_type__ == 3)
    return mell_DMconst(k,N);
  return 0;
}
//
dcomplex S1(dcomplex N) {
  return 4.*CA*mellD(1,N);
}
dcomplex S2(dcomplex N) {
  dcomplex res = ( (2./3.*_nf - 11./3.*CA)*CA * mellD(2,N)
		  + (CA*CA*(67./9. - 2.*ZETA2) - CA*10./9.*_nf) * mellD(1,N)
		  + (CA*CA*(-101./27. + 11./3.*ZETA2 + 7./2.*ZETA3) + _nf*CA*(14./27.-2./3.*ZETA2)) * mellD(0,N)
		  );
  return res;
}
dcomplex S1_sq(dcomplex N) {
  return 16*CA*CA*(mellD(3,N) - 2.*ZETA2*mellD(1,N) + 2.*ZETA3*mellD(0,N) - ZETA2*ZETA2/10.);
}
dcomplex S1_cube(dcomplex N) {
  return 64*CA*CA*CA*( 3./4.*mellD(5,N) - 6.*ZETA2*mellD(3,N) + 15.*ZETA3*mellD(2,N) -3./2.*ZETA2*ZETA2*mellD(1,N)
		       +(18.*ZETA5-12.*ZETA2*ZETA3)*mellD(0,N) + 5*ZETA3*ZETA3 -29./35.*ZETA2*ZETA2*ZETA2);
}
dcomplex S1_S2(dcomplex N) {
  double b22 = (2./3.*_nf - 11./3.*CA)*CA;
  double b21 = CA*CA*(67./9. - 2.*ZETA2) - CA*10./9.*_nf;
  double b20 = CA*CA*(-101./27. + 11./3.*ZETA2 + 7./2.*ZETA3) + _nf*CA*(14./27.-2./3.*ZETA2);
  return 4*CA*( 5./6.*b22*mellD(4,N) + b21*mellD(3,N)
		+ (3./2.*b20-3.*ZETA2*b22) *mellD(2,N)
		+ (6.*ZETA3*b22-2.*ZETA2*b21) *mellD(1,N)
		+ (-ZETA2*ZETA2*b22+2.*ZETA3*b21-ZETA2*b20) *mellD(0,N)
		+ (4.*ZETA5-2.*ZETA2*ZETA3)*b22 -1./10.*ZETA2*ZETA2*b21 +ZETA3*b20
		);
}
//
dcomplex FO_2_log_PL(dcomplex N) {
  dcomplex value;
  value = ( 72. * mellD(3,N)
	    + (2.*_nf - 33.) * mellD(2,N)
	    + (67. - 90.*ZETA2 - 10./3.*_nf) * mellD(1,N)
	    + (-101./3. + 33.*ZETA2 + 351./2.*ZETA3 + _nf*(14./9.-2.*ZETA2)) * mellD(0,N)
	    );
  return value;
}
dcomplex FO_3_log_PL(dcomplex N) {
  double CA3 = CA*CA*CA;
  dcomplex value;
  value = ( 8*CA3 * mellD(5,N)
	    + (-110./9.*CA3 + 20/9.*CA*CA*_nf) * mellD(4,N)
	    + (CA3*(925./27. - 56*ZETA2) - 164./27.*CA*CA*_nf + 4./27.*CA*_nf*_nf) * mellD(3,N)
	    + (CA3 * (-1051./27. + 187./3*ZETA2 + 181*ZETA3)
	       + CA*CA*_nf * (457./2./27. - 34./3.*ZETA2)
	       + 0.5*CA*CF*_nf - 10./27.*CA*_nf*_nf ) * mellD(2,N)
	    + (CA3 * (30569./8./81. - 152./3.*ZETA2 - 352./3.*ZETA3 - 154./5.*ZETA2*ZETA2)
	       + CA*CA*_nf * (-4211./4./81. + 94./9.*ZETA2 + 46./3.*ZETA3)
	       - CA*CF*_nf * (63./8.-6.*ZETA3)
	       + CA*_nf*_nf * (25./81.-4./9.*ZETA2) ) * mellD(1,N)
	    + (CA3 * (-297029./32./729. + 8563./4./81.*ZETA2 + 8941./4./27.*ZETA3 + 253./4./15.*ZETA2*ZETA2
		      - 725./2./3.*ZETA2*ZETA3 + 186.*ZETA5)
	       + CA*CA*_nf * (31313./16./729. - 2173./4./81.*ZETA2 - 475./4./9.*ZETA3 - 17./2./15*ZETA2*ZETA2)
	       + CA*CF*_nf * (1711./32./27. - 0.5*ZETA2 - 19./2./9.*ZETA3 - 1./5.*ZETA2*ZETA2)
	       - CA*_nf*_nf * (58./729. - 10./27.*ZETA2 - 5./27.*ZETA3) ) * mellD(0,N)
	    );
  return value;
}
//
dcomplex APimprovement(dcomplex f(dcomplex), dcomplex N) {
  dcomplex res = 0;
  if(APtype== 0) res = f(N);                                           // Ag(1)
  if(APtype== 1) res = f(N+1);                                         // Ag expanded to first order
  if(APtype== 2) res = 2*f(N  ) -3*f(N+1) +2*f(N+2);                   // full Ag expanded to second order
  if(APtype==-1) res =   f(N-1) -2*f(N  ) +3*f(N+1) -2*f(N+2) +f(N+3); // full Ag
  if(APtype==-2) res =(3*f(N+1) -2*f(N+2) +  f(N+3))/2.;               // soft Ags
  if(APtype==-3) res =   f(N-1);                                       // small-x
  return res;
}
double APimprovement(double z) {
  double res = 0;
  if(APtype== 0) res = 1;                                       // Ag(1)
  if(APtype== 1) res = z;                                       // Ag expanded to first order
  if(APtype== 2) res = (2-3*z+2*z*z);                           // full Ag expanded to second order
  if(APtype==-1) res = (1./z-2+3*z-2*z*z+z*z*z);                // full Ag
  if(APtype==-2) res = (3*z-2*z*z+z*z*z/2);                     // soft Ags
  if(APtype==-3) res = 1./z;                                    // small-x
  return res;
}




// Expansion from resummation (with soft scale dependence)
int LL_order = 3;
dcomplex asS1(dcomplex N) {
  const double as = 1; // fake, just to keep trac of the order
  dcomplex L1 = mellD(0,N);
  dcomplex L2 = 2*mellD(1,N) + 2*EulerGamma*L1;
  dcomplex value = as*2*L2*rA1;
  if(LL_order > 0) value += as*L1*4*(log(muM/muF)-EulerGamma)*rA1;
  return value;
}
dcomplex as2S2(dcomplex N) {
  const double as = 1; // fake, just to keep trac of the order
  double bb0 = b0/M_PI;
  dcomplex L1 = mellD(0,N);
  dcomplex L2 = 2*mellD(1,N) + 2*EulerGamma*L1;
  dcomplex L3 = 3*mellD(2,N)-3*ZETA2*L1 + 3*EulerGamma*L2 - 3*EulerGamma*EulerGamma*L1;
  dcomplex value = - as*as*4*L3*rA1*bb0/3.;
  if(LL_order > 0) value += as*as*L2* 2*( rA1*2*(EulerGamma-log(muM/muR))*bb0 + rA2 );
  if(LL_order > 1) value += as*as*L1* ( rD2 - 4.*rA2*EulerGamma - 4.*rA1*(EulerGamma*EulerGamma+ZETA2)*bb0
					+ 8.*rA1*EulerGamma*log(muM/muR)*bb0 +4.*rA2*log(muM/muF)
					- 4.*rA1*(pow(log(muM/muR),2.)-pow(log(muF/muR),2.))*bb0 );
  return value;
}
dcomplex as3S3(dcomplex N) {
  const double as = 1; // fake, just to keep trac of the order
  double bb0 = b0/M_PI;
  double bb1 = b1/b0/M_PI;
  dcomplex L1 = mellD(0,N);
  dcomplex L2 = 2*mellD(1,N) + 2*EulerGamma*L1;
  dcomplex L3 = 3*mellD(2,N)-3*ZETA2*L1 + 3*EulerGamma*L2 - 3*EulerGamma*EulerGamma*L1;
  dcomplex L4 = 4*mellD(3,N)-12*ZETA2*mellD(1,N)+8*ZETA3*L1 + 4*EulerGamma*L3 - 6*EulerGamma*EulerGamma*L2 + 4*EulerGamma*EulerGamma*EulerGamma*L1;
  dcomplex value = as*as*as*L4*4*rA1*bb0*bb0/3.;
  if(LL_order > 0) value += -as*as*as*L3*bb0*4./3.* (2*rA2 + rA1*(bb1 + 4*bb0*(EulerGamma-log(muM/muR))) );
  if(LL_order > 1) value += as*as*as*L2*2* ( rA3 + bb0*(-rD2 + 4*rA2*EulerGamma
						      + 2.*rA1*(bb1*EulerGamma + 2*bb0*(EulerGamma*EulerGamma+ZETA2)))
					     - 2*bb0*(2*rA2 + rA1*(bb1+4*bb0*EulerGamma))*log(muM/muR)
					     + 4*rA1*bb0*bb0*pow(log(muM/muR),2.) );
  if(LL_order > 2) value += as*as*as*L1/3.* (3*rD3 - 12*rA3*EulerGamma + 12*bb0*rD2*EulerGamma - 24*rA2*bb0*pow(EulerGamma,2)
					     - 12*rA1*bb0*bb1*pow(EulerGamma,2) - 16*rA1*pow(bb0,2)*pow(EulerGamma,3) - 24*rA2*bb0*ZETA2
					     - 12*rA1*bb0*bb1*ZETA2 - 48*rA1*pow(bb0,2)*EulerGamma*ZETA2 - 32*rA1*pow(bb0,2)*ZETA3
					     + 12*(rA3 + bb0*(-rD2 + 4*rA2*EulerGamma + 2*rA1*(bb1*EulerGamma + 2*bb0*(pow(EulerGamma,2) + ZETA2))))*log(muM/muR)
					     - 12*bb0*(2*rA2 + rA1*(bb1 + 4*bb0*EulerGamma))*pow(log(muM/muR),2)
					     + 16*rA1*pow(bb0,2)*pow(log(muM/muR),3) - 12*rA3*log(muF/muR)
					     + 12*bb0*(2*rA2 + rA1*bb1)*pow(log(muF/muR),2) - 16*rA1*pow(bb0,2)*pow(log(muF/muR),3));
  return value;
}
//
dcomplex g0expS_order1(dcomplex N) {
  const double as = 1; // fake, just to keep trac of the order
  dcomplex value;
  value = APimprovement(asS1,N);
  if(LL_order > 0) value += g0bar1*as;
  return value * M_PI;
}
dcomplex g0expS_order2(dcomplex N) {
  const double as = 1; // fake, just to keep trac of the order
  dcomplex value;
  value = pow(APimprovement(asS1,N),2.)/2. + APimprovement(as2S2,N);
  if(LL_order > 0) value += g0bar1*as*APimprovement(asS1,N);
  if(LL_order > 1) value += g0bar2*as*as;
  return value * M_PI*M_PI;
}
dcomplex g0expS_order3(dcomplex N) {
  const double as = 1; // fake, just to keep trac of the order
  dcomplex value;
  value = pow(APimprovement(asS1,N),3.)/6. + APimprovement(asS1,N)*APimprovement(as2S2,N) + APimprovement(as3S3,N);
  if(LL_order > 0) value += g0bar1*as*( pow(APimprovement(asS1,N),2.)/2. + APimprovement(as2S2,N) );
  if(LL_order > 1) value += g0bar2*as*as*APimprovement(asS1,N);
  if(LL_order > 2) value += g0bar3*as*as*as;
  return value * M_PI*M_PI*M_PI;
}
bool dFMMV = false;
bool NextToSoft = false;
dcomplex g0expS(dcomplex N) {
  dcomplex res = 0;
  if(order == 1) res = g0expS_order1(N);
  if(order == 2) res = g0expS_order2(N);
  if(order == 3) res = g0expS_order3(N);
  if(NextToSoft) {
    double W1 = (finite_mt ? _W1t : _W1);
    double W2 = (finite_mt ? _W2t : _W2);
    dcomplex L, dc1, dc2, dc3;
    if(__log_type__==1 && APtype>0) {
      L = psi0(N);
      dc1 = 0;
      dc2 = (31.4967*L*L +36.8633*L -6.85889)/N;
      dc3 = (189.*L*L*L*L + 546.708*L*L*L + 1197.47*L*L + 782.976*L -245.534)/N;
    } else {
      //cout << "Next-to-soft not implemented for this choice of logs" << endl;
    }
    if(order == 1) res += dc1;
    if(order == 2) res += dc2 + W1*dc1;
    if(order == 3) res += dc3 + W1*dc2 + W2*dc1;
  }
  if(dFMMV) {
    if(__log_type__==3 && APtype==0) {
      dcomplex L = log(N);
      dcomplex c1 =   6.*(L + EulerGamma)/N;
      dcomplex c2 =  36.*(L*L*L + 2.926*L*L + 5.970*L + 2.007)/N;
      dcomplex c3 = 108.*(L*L*L*L*L + 5.701*L*L*L*L + 18.9*L*L*L + 46.*L*L + 18.*L + 9.)/N;
      double W1 = (finite_mt ? _W1t : _W1);
      double W2 = (finite_mt ? _W2t : _W2);
      if(order == 1) res += c1;
      if(order == 2) res += c2 + W1*c1;
      if(order == 3) res += c3 + W1*c2 + W2*c1;
    } else {
      cout << "dFMMV not implemented for our result" << endl;
      //if(order == 2) res += pow(CA/4.,2)* 56*mell_L(0,2,N);
      //if(order == 3) res += pow(CA/4.,3)*448*mell_L(0,4,N);
    }
  }
  return res;
}


const double MPbeta = 1;
const unsigned int MPgamma = 0;
const unsigned int Nchebyshev = 13;
double ChebCoeff_gg[Nchebyshev+1];
double ChebCoeff_qg[Nchebyshev+1];
double ChebCoeff_SS[Nchebyshev+1];
double ChebCoeff_TT[Nchebyshev+1];
double ChebCoeff_VV[Nchebyshev+1];
dcomplex LuminosityN(dcomplex N, double *ChebCoeff) {
  dcomplex value = 0;
  for(unsigned int i=0; i<=Nchebyshev; i++) {
    for(unsigned int j=0; j<=MPgamma; j++) {
      value += ChebCoeff[i] * pow(-1.,j) * gsl_sf_choose(MPgamma,j) / pow(N+j-MPbeta,i+1.);
    }
  }
  return value;
}
dcomplex lum_times_g0expS(dcomplex N) {
  return g0expS(N) * LuminosityN(N, ChebCoeff_gg);
}

// exact muF dependence
#include "fixed-order_Nspace.cc"
dcomplex muFdep_3(dcomplex N) {
  double mu = muM;
  double lf = -2.*log(_muF_mu_ratio);
  // scale dependence
  //dcomplex C1qg = CF*mell_L(0,1,N) + CF/2./N;
  //double xht = pow(mu/mtop,2);
  //dcomplex C2qg = 367./54.*mell_L(0,3,N) + (85./36.+_nf/18.)*mell_L(0,2,N) + (341./18.-50./9.*ZETA2-_nf*2./3.)*mell_L(0,1,N) + (11./27.+29./6.*ZETA2+311./18.*ZETA3+_nf*13./81.)
  //  + xht*(136./405.*mell_L(0,1,N) + 68./405.)
  //  + xht*xht*(3553./85050.*mell_L(0,1,N) + 3553./170100.);
  dcomplex C1qg = NFO_1_qg(N,mu);
  dcomplex C2qg = NFO_2_qg(N,mu);
  //
  //dcomplex C1gg = g0expS_order1(N);
  dcomplex C1gg = NFO_1_gg(N,mu);
  dcomplex C2gg = NFO_2_gg(N,mu);
  //
  dcomplex C1qqb = NFO_1_qqb(N,mu);
  dcomplex C2qq  = NFO_2_qq (N,mu);
  dcomplex C2qqb = NFO_2_qqb(N,mu);
  dcomplex C2qqp = NFO_2_qqp(N,mu);
  // singlet
  dcomplex C1SS = C1qqb/(2.*_nf);
  dcomplex C2SS = (C2qq + C2qqb + 2*(_nf-1.)*C2qqp)/(2.*_nf);
  // non-singlet
  dcomplex C1TT = C1qqb/2.;
  dcomplex C2TT = (C2qq + C2qqb - 2*C2qqp)/2.;
  dcomplex C1VV = -C1qqb/2.;
  dcomplex C2VV = (C2qq - C2qqb)/2.;
  //
  dcomplex g0gg = M_PI*gamma0gg(N,_nf);
  dcomplex g0gq = M_PI*gamma0gq(N);
  dcomplex g0qg = M_PI*gamma0qg(N,_nf);
  dcomplex g0qq = M_PI*gamma0qq(N);
  //
  gamma1sums g1s;
  sums(N,g1s);
  dcomplex g1gg = M_PI*M_PI*gamma1SGgg(N,_nf,g1s);
  dcomplex g1gq = M_PI*M_PI*gamma1SGgq(N,_nf,g1s);
  dcomplex g1qg = M_PI*M_PI*gamma1SGqg(N,_nf,g1s);
  dcomplex g1qq = M_PI*M_PI*gamma1SGqq(N,_nf,g1s); // this is the singlet = NSplus + PS 
  dcomplex g1mn = M_PI*M_PI*gamma1NSminus(N,_nf,g1s);
  dcomplex g1pl = M_PI*M_PI*gamma1NSplus(N,_nf,g1s);
  //
  //dcomplex g2gg = M_PI*M_PI*M_PI*gamma2SGgg(N,_nf);
  //dcomplex g2gq = M_PI*M_PI*M_PI*gamma2SGgq(N,_nf);
  dcomplex g2gg, g2gq, g2qg, g2ps;
  p2ijp(g2ps, g2qg, g2gq, g2gg, N, _nf);
  g2gg /= 64.;
  g2gq /= 64.;
  g2qg /= 64.;
  g2ps /= 64.;
  //dcomplex g2pl, g2mn, g2vl;
  //p2nsp(g2pl, g2mn, g2vl, N, _nf);
  //g2pl /= 64.;
  //g2mn /= 64.;
  //g2vl /= 64.;
  //
  // full
  dcomplex FO3gg = 0;
  FO3gg += (C2gg*(8*b0 - 4*g0gg) + C1gg*(6*b1 - 4*g1gg) - 4*(-b2 + C2qg*g0qg + C1qg*g1qg + g2gg))*(-lf/2.)
    + 4*(-5*b1*g0gg + 3*C1qg*g0gg*g0qg + C1SS*pow(g0qg,2) + C1gg*(6*pow(b0,2) - 7*b0*g0gg + 2*pow(g0gg,2) + g0gq*g0qg) + C1qg*g0qg*g0qq + 
	 b0*(7*b1 - 7*C1qg*g0qg - 6*g1gg) + 4*g0gg*g1gg + g0qg*g1gq + g0gq*g1qg)*pow(lf/2.,2)
    + (8*(12*pow(b0,3) - 26*pow(b0,2)*g0gg - 4*pow(g0gg,3) - 5*g0gg*g0gq*g0qg + 9*b0*(2*pow(g0gg,2) + g0gq*g0qg) - g0gq*g0qg*g0qq)*pow(-lf/2.,3))/3.;
  //
  dcomplex FO3qg = 0;
  //FO3qg = pow(psi0(N+1),5)/N *CF*(18*CF*CF+100./3.*CF*CA+230./3.*CA*CA)/pow(4.,3); // Vogt prediction
  //FO3qg = 1283./72.*mell_L(0,5,N) - 5215./2592.*mell_L(0,4,N) -114.569021*mell_L(0,3,N) +513.56298*mell_L(0,2,N) -313.98523*mell_L(0,1,N) +204.62079/N;
  FO3qg += 2*(C2gg*g0gq + C2SS*g0qg + C2qg*(-4*b0 + g0gg + g0qq) + C1gg*g1gq + C1SS*g1qg + C1qg*(-3*b1 + g1gg + g1qq) + g2gq)*lf/2.
    + 2*(-5*b1*g0gq - 7*C1SS*b0*g0qg + C1SS*g0gg*g0qg + 3*C1SS*g0qg*g0qq + C1gg*g0gq*(-7*b0 + 3*g0gg + g0qq)
	 + C1qg*(12*pow(b0,2) + pow(g0gg,2) + 4*g0gq*g0qg + 2*g0gg*g0qq + pow(g0qq,2) - 7*b0*(g0gg + g0qq))
	 + 3*g0gq*g1gg - 6*b0*g1gq + 3*g0gg*g1gq + g0qq*g1gq + g0gq*g1qq)*pow(lf/2.,2)
    + 4*g0gq*(26*pow(b0,2) + 7*pow(g0gg,2) + 4*g0gq*g0qg + 4*g0gg*g0qq + pow(g0qq,2) - 9*b0*(3*g0gg + g0qq))*pow(lf/2.,3)/3.;
  //
  dcomplex FO3SS = 0;
  FO3SS += (8*C2SS*b0 + 6*C1SS*b1 - 4*C2qg*g0gq - 4*C2SS*g0qq - 4*C1qg*g1gq - 4*C1SS*g1qq)*(-lf/2.)
    + 4*(C1SS*(6*pow(b0,2) + g0gq*g0qg - 7*b0*g0qq + 2*pow(g0qq,2)) + g0gq*(C1gg*g0gq + C1qg*(-7*b0 + g0gg + 3*g0qq) + 2*g1gq))*pow(lf/2.,2)
    + 8*pow(g0gq,2)*(3*b0 - g0gg - g0qq)*pow(-lf/2.,3);
  //
  dcomplex FO3TT = 0;
  FO3TT += (4*C2TT*b0 + 3*C1TT*b1 - 2*C2TT*g0qq - 2*C1TT*g1pl)*(-lf)
    + (C1TT*(6*pow(b0,2) - 7*b0*g0qq + 2*pow(g0qq,2)))*pow(lf,2);
  //
  dcomplex FO3VV = 0;
  FO3VV += (4*C2VV*b0 + 3*C1VV*b1 - 2*C2VV*g0qq - 2*C1VV*g1mn)*(-lf)
    + (C1VV*(6*pow(b0,2) - 7*b0*g0qq + 2*pow(g0qq,2)))*pow(lf,2);
  //
  dcomplex res = FO3gg * LuminosityN(N, ChebCoeff_gg);
  if(__all_channels__) res += FO3qg * LuminosityN(N, ChebCoeff_qg) + FO3SS * LuminosityN(N, ChebCoeff_SS) + FO3TT * LuminosityN(N, ChebCoeff_TT) + FO3VV * LuminosityN(N, ChebCoeff_VV);
  return res;
}
dcomplex muFdep_2(dcomplex N) {
  double mu = muM;
  double lf = -2.*log(_muF_mu_ratio);
  double mufrat = _muF_mu_ratio; _muF_mu_ratio = 1;
  // scale dependence
  //dcomplex C1qg = CF*mell_L(0,1,N) + CF/2./N;
  //double xht = pow(mu/mtop,2);
  //dcomplex C2qg = 367./54.*mell_L(0,3,N) + (85./36.+_nf/18.)*mell_L(0,2,N) + (341./18.-50./9.*ZETA2-_nf*2./3.)*mell_L(0,1,N) + (11./27.+29./6.*ZETA2+311./18.*ZETA3+_nf*13./81.)
  //  + xht*(136./405.*mell_L(0,1,N) + 68./405.)
  //  + xht*xht*(3553./85050.*mell_L(0,1,N) + 3553./170100.);
  dcomplex C1qg = NFO_1_qg(N,mu);
  //
  //dcomplex C1gg = g0expS_order1(N);
  dcomplex C1gg = NFO_1_gg(N,mu);
  //
  dcomplex C1qqb = NFO_1_qqb(N,mu);
  // singlet
  dcomplex C1SS = C1qqb/(2.*_nf);
  // non-singlet
  dcomplex C1TT = C1qqb/2.;
  dcomplex C1VV = -C1qqb/2.;
  //
  dcomplex g0gg = M_PI*gamma0gg(N,_nf);
  dcomplex g0gq = M_PI*gamma0gq(N);
  dcomplex g0qg = M_PI*gamma0qg(N,_nf);
  dcomplex g0qq = M_PI*gamma0qq(N);
  //
  gamma1sums g1s;
  sums(N,g1s);
  dcomplex g1gg = M_PI*M_PI*gamma1SGgg(N,_nf,g1s);
  dcomplex g1gq = M_PI*M_PI*gamma1SGgq(N,_nf,g1s);
  //
  // full
  dcomplex FO2gg = 0;
  FO2gg += (C1gg*(6*b0 - 4*g0gg) +4*b1 - 4*C1qg*g0qg -4*g1gg)*(-lf/2.)
    + (3*pow(b0,2) - 5*b0*g0gg + 2*pow(g0gg,2) + g0gq*g0qg)*pow(-lf,2);
  //
  dcomplex FO2qg = 0;
  FO2qg += (C1gg*g0gq + C1SS*g0qg + C1qg*(-3*b0 + g0gg + g0qq) + g1gq)*lf
    + g0gq*(-5*b0 + 3*g0gg + g0qq)*pow(lf,2)/2.;
  //
  dcomplex FO2SS = 0;
  FO2SS += (C1SS*(3*b0 - 2*g0qq) - 2*C1qg*g0gq)*(-lf) + g0gq*g0gq*lf*lf;
  //
  dcomplex FO2TT = 0;
  FO2TT += C1TT*(3*b0 - 2*g0qq)*(-lf);
  //
  dcomplex FO2VV = 0;
  FO2VV += C1VV*(3*b0 - 2*g0qq)*(-lf);
  //
  dcomplex res = FO2gg * LuminosityN(N, ChebCoeff_gg);
  if(__all_channels__) res += FO2qg * LuminosityN(N, ChebCoeff_qg) + FO2SS * LuminosityN(N, ChebCoeff_SS) + FO2TT * LuminosityN(N, ChebCoeff_TT) + FO2VV * LuminosityN(N, ChebCoeff_VV);
  _muF_mu_ratio = mufrat;
  return res;
}
dcomplex muFdep(dcomplex N) {
  //if(order==2) return muFdep_2(N);
  if(order==3) return muFdep_3(N);
  return 0;
}
double MellinInversionIntegrand(double t, void *p) {
  double x = *(double*)p;
  dcomplex N = MPbeta + 3. + (1.-I)*log(t);
  dcomplex res = lum_times_g0expS(N);
  if(exactmuFdep) res += muFdep(N);
  //if(order==2 && __exact__) res = muFdep(N);
  res *= (I-1.) * exp(-N*log(x)) /t;
  return imag(res)/M_PI;
}
double MellinInversion(double tau, double *err=NULL) {
  double res=0, prec = 1.e-3;
  //gsl_error_handler_t *old_handler = gsl_set_error_handler (NULL);
  //gsl_set_error_handler_off();
  res = gauss(MellinInversionIntegrand, 0., 1., prec, err, &tau);
  //gsl_set_error_handler (old_handler);
  double lgg = Lum(tau, muF);
  *err /= lgg;
  return res/lgg;
}













// Fixed-order (NLO)
///////////////////////////////////////
double FO_1_log(double z, double D(int,double)) {
  double lf = -2.*log(_muF_mu_ratio);
  return FO_1_log_PL(z,D) + 2*CA*D(0,z)*lf;
}
double FO_1_log_APimproved(double z, double D(int,double)) {
  return APimprovement(z)*FO_1_log(z,D);
}
double FO_1_Nsoftcoll_impr(double z) {
  return 2.*CA*(gsl_sf_psi(1.)-log(log(1./z)));
}
double FO_1_delta_const() {
  double W1 = (finite_mt ? _W1t : _W1);
  double c = _cd1 + W1;
  //double lrf = 2.*log(_muR_muF_ratio);
  //c += 2*b0*lrf;
  return c;
}
// finite mt R_gg term
double Rgg_zspace(double z, double mu) {
  double z0 = (mu<200 ? 0.0002 : 0.01);
  double c1 = sxc->c1marzani(mu,mtop);
  //double c1 = z0*Rgg(z0, mu*mu/z0) - 2.*log(z0);
  double rgg = (z<z0 ? (c1+2.*log(z))/z : Rgg(z,mu*mu/z));
  return rgg;
}
double Rqg_zspace(double z, double mu) {
  double z0 = 0.000002;  // apparently it behaves better than Rgg,
  double c1 = sxc->c1marzani(mu,mtop); // so the asymptotic is not really needed
  double rqg = (z<z0 ? (c1/2.*CF + CF*log(z))/z : Rqg(z,mu*mu/z));
  return rqg;
}
double Rqqbar_zspace(double z, double mu) {
  // This has no numerical instabilities
  return Rqqbar(z,mu*mu/z);
}
double FO_1_nonlog(double z, double mu) {
  double lf = -2.*log(_muF_mu_ratio);
  double smallx = (finite_mt ? CA*Rgg_zspace(z, mu) : - 11./2.*pow(1.-z,3.)/z );
  double nonlog = CA * 2.*( (1./z-2.+z*(1.-z))*(2.*log(1.-z)-log(z)) -log(z)/(1.-z) ) + smallx;
  return nonlog + 2.*CA*(1./z-2.+z-z*z)*lf;
}
double FO_1_hard(double z, double mu) {
  double lf = -2.*log(_muF_mu_ratio);
  double c1 = sxc->c1marzani(mu,mtop);
  c1 += 2.*lf;
  double e01 = CA;
  //double e00 = (-11.*CA+2.*_nf*(2.*CF/CA-1.))/12.; // delta term in z-space, cancelled by damping
  double hard = c1*e01/z;// + c1*e00;
  return (hard - 12.*MomConsCoeff[0]) * pow(1-z, 2);
}
double FO_1_qg(double z, double mu) {
  double lf = -2.*log(_muF_mu_ratio);
  double Pgq = CF*(1.+pow(1.-z,2))/z/2.;
  double smallx = (finite_mt ? Rqg_zspace(z, mu) : 2./3.*z - pow(1.-z,2)/z );
  return Pgq*(2.*log(1-z)-log(z)+lf) + smallx;
}
double FO_1_qqb(double z, double mu) {
  return (finite_mt ? Rqqbar_zspace(z, mu) : 32./27.*pow(1-z,3)/z );
}
//




// Fixed-order (NNLO)
///////////////////////////////////////
dcomplex FO_2_log(dcomplex N) {
  double W1 = (finite_mt ? _W1t : _W1);
  return FO_2_log_PL(N) + W1*S1(N);
}
double FO_2_log(double z, double D(int,double)) {
  double lf = -2.*log(_muF_mu_ratio);
  double W1 = (finite_mt ? _W1t : _W1);
  double Clog = FO_2_log_PL(z,D) + W1*FO_1_log_PL(z,D);
  Clog += ( 108*lf * D(2,z)
	    + (36.*lf*lf - (33-2*_nf)*lf) * D(1,z)
	    + (-(33-2*_nf)/4*lf*lf + (2.*CA*W1 + 67./2.-5./3.*_nf-5.*CA*CA*ZETA2)*lf) * D(0,z) );
  //double lrf = 2.*log(_muR_muF_ratio);
  //Clog += 3*b0*FO_1_log(z,D)*lrf;
  return Clog;
  //if(__exact__ && finite_mt) return gg_nnlo_mt_distr(z, mu, mtop, _muF_mu_ratio); // no longer needed
}
dcomplex FO_2_log_APimprovement(dcomplex N, void *par) {
  return 0.5*(pow(APimprovement(S1,N),2) - APimprovement(S1_sq,N));
}
double FO_2_log_APimprovement_soft1(double z) {
  double piece = (Li3(z) - ZETA3  + 0.5*log(z)*(-ZETA2 + 1./6.*pow(log(z),2) - Li2(z) - 0.5*log(z)*log(1-z)))/(1.-z);
  return 8*CA*CA*z*piece;
}
double FO_2_log_APimprovement_soft2(double z) {
  double piece = (Li3(z) - ZETA3  + 0.5*log(z)*(-ZETA2 + 1./6.*pow(log(z),2) - Li2(z) - 0.5*log(z)*log(1-z)))/(1.-z);
  return 8*CA*CA*(-4.+4.*z+4.*ZETA2-12.*z*ZETA2
		  +8.*log(1.-z) + 2.*(-1.+ZETA2+z*(3.+ZETA2))*log(z) +2.*z*pow(log(z),2)
		  -1./12.*(1.+z)*pow(log(z),3) - 4.*log(1-z)* (ZETA2+z*(2.+ZETA2)+2.*(-1+z)*log(z))
		  +2.*pow(log(1-z),2)*(-4.+4.*z+log(z)+z*log(z)) + 2.*(1.+z)*(2.+2.*log(1-z)-log(z))*Li2(z)
		  +4.*(1+z)*Li3(1-z) + 2.*(1+z)*Li3(z) - 2.*(1+z)*ZETA3
		  + (2.-3.*z+2.*z*z)*piece);
}
double FO_2_log_APimproved(double z, double D(int,double)) {
  double res = APimprovement(z)*FO_2_log(z,D);
  //if(APimprExponent) res += InverseMellin(z,FO_2_log_APimprovement,NULL);
  if(APimprExponent) {
    if     (APtype == 1 && __log_type__ == 1) res += FO_2_log_APimprovement_soft1(z);
    else if(APtype == 2 && __log_type__ == 1) res += FO_2_log_APimprovement_soft2(z);
    else res += InverseMellin(z,FO_2_log_APimprovement,NULL);
  }
  return res;
}
dcomplex FO_2_Nsoftcoll_impr(dcomplex N, void *par) {
  return 2.*CA*log(N)/N*S1(N);
}
double FO_2_Nsoftcoll_impr(double z) {
  return InverseMellin(z,FO_2_Nsoftcoll_impr,NULL);
}
double FO_2_delta_const(double mu) {
  double lf = -2.*log(_muF_mu_ratio);
  double W1 = (finite_mt ? _W1t : _W1);
  double W2 = (finite_mt ? _W2t : _W2);
  double c = _cd2 + W2 + _cd1*W1;
  c += -18.*ZETA2*lf*lf + (-3./4. + 171./2.*ZETA3 + 1./4.*_nf - (33.-2.*_nf)/2.*ZETA2 - W1*(33.-2.*_nf)/12.)*lf;
  //if(finite_mt) c = gg_nnlo_mt_delta(mu, mtop, _muF_mu_ratio); // no longer needed (checked)
  //
  //double lrf = 2.*log(_muR_muF_ratio);
  //double muRF = _muR_muF_ratio;  _muR_muF_ratio = 1.;
  //c += (2*b1+3*b0*FO_1_delta_const())*lrf + 3*b0*b0*lrf*lrf;
  //_muR_muF_ratio = muRF;
  return c;
}
// sigma(z) = z C(z)
// C(z) = [D(z)]_+ + nonlog(z)
// sigma(z) = [D(z)]_+ - (1-z)D(z) + z*nonlog(z)
//          = [D(z)]_+ + reg(z)
// nonlog(z) = [ reg(z) + (1-z)D(z) ] /z
double FO_2_nonlog(double z, double mu) {
  double nonlog;
  if(!finite_mt) nonlog = ( C2reg_pointlike(z, _nf, _muF_mu_ratio) + (1.-z)*FO_2_log(z,Dk) ) /z;
  else {
    double lf = -2.*log(_muF_mu_ratio);
    double c1 = sxc->c1marzani(mu,mtop);
    double c2 = sxc->c2marzani(mu,mtop);
    c2  += (c1+lf)*lf*2.;
    if(mu<2*mtop)
      nonlog = ( gg_nnlo_mt_reg(z, mu, mtop, _muF_mu_ratio) + smallx_match_z(z, c2) + (1.-z)*gg_nnlo_mt_distr(z, mu, mtop, _muF_mu_ratio) ) /z;
    else {
      //nonlog = ( gg_nnlo_infmt_reg(z, mu, mtop, _muF_mu_ratio) + smallx_match_z(z, c2) + (1.-z)*gg_nnlo_infmt_distr(z, mu, mtop, _muF_mu_ratio) ) /z;
      nonlog = ( gg_nnlo_infmt_reg(z, mu, mtop, _muF_mu_ratio) + (1.-z)*gg_nnlo_infmt_distr(z, mu, mtop, _muF_mu_ratio) ) /z;
      //finite_mt = false;  nonlog = ( C2reg_pointlike(z, _nf, _muF_mu_ratio) + (1.-z)*FO_2_log(z,Dk) ) /z;  finite_mt = true;
      double nonlog1 = CA * 2.*( (1./z-2.+z*(1.-z))*(2.*log(1.-z)-log(z)) -log(z)/(1.-z) );
      nonlog += (_W1t-_W1)*nonlog1;
    }
  }
  //double lrf = 2.*log(_muR_muF_ratio);
  //nonlog += 3*b0*FO_1_nonlog(z,mu)*lrf;
  return nonlog;
}
//
double FO_2_hard(double z, double mu) {
  //double hard = SF(z, mu, FO_2_sx_NLL_RC); // ok checked
  //double lrf = 2.*log(_muR_muF_ratio);
  //hard += 3*b0*FO_1_hard(z,mu)*lrf;
  // z-space version: faster
  double lf = -2.*log(_muF_mu_ratio);
  double c10 = sxc->c10(mu,mtop);
  double c20 = sxc->c20(mu,mtop);
  double c11 = sxc->c11(mu,mtop);
  double c1 = 2*c10;
  double c2 = 2.*c20+c11;
  c20 += (c1+lf)*lf/2.;
  c2  += (c1+lf)*lf*2.;
  c1  += 2*lf;
  double e01 = CA;
  double e00 = (-11.*CA+2.*_nf*(2.*CF/CA-1.))/12.;
  double e11 = _nf*(26.*CF-23.*CA)/36.;
  double hard = -c2*e01*e01*log(z)/z + (2*c2*e00*e01-2*c20*b0*e01+c1*e11)/z;
  //
  return (hard - 12.*MomConsCoeff[1]) * pow(1-z, 2);
}
double FO_2_qg(double z, double mu) {
  double res;
  if(!finite_mt) res = C2reg_pointlike_qg(z, _nf, _muF_mu_ratio) /z;
  else {
    double lf = -2.*log(_muF_mu_ratio);
    double c10 = sxc->c10(mu,mtop);
    double c20 = sxc->c20(mu,mtop);
    double c11 = sxc->c11(mu,mtop);
    double c1 = 2*c10;
    double c2 = 2.*c20+c11;
    c2  += (c1+lf)*lf*2.;
    c20 += (c1+lf)*lf/2.;
    if(mu<2*mtop)
      res = ( qg_nnlo_mt(z, mu, mtop, _muF_mu_ratio) + smallx_match_z(z, (c2-c20)*CF/CA) ) /z;
    else {
      res = qg_nnlo_infmt(z, mu, mtop, _muF_mu_ratio) /z;
      double Pgq = CF*(1.+pow(1.-z,2))/z/2.;
      double res1 = Pgq*(2.*log(1-z)-log(z));
      res += (_W1t-_W1)*res1;
    }
  }
  return res;
}
double FO_2_qqb(double z, double mu) {
  double res;
  if(!finite_mt) res = C2reg_pointlike_qqb(z, _nf, _muF_mu_ratio) /z;
  else {
    double lf = -2.*log(_muF_mu_ratio);
    double c10 = sxc->c10(mu,mtop);
    double c20 = sxc->c20(mu,mtop);
    double c11 = sxc->c11(mu,mtop);
    double c1 = 2*c10;
    double c2 = 2.*c20+c11;
    c2  += (c1+lf)*lf*2.;
    c20 += (c1+lf)*lf/2.;
    if(mu<2*mtop)
      res = ( qu_nnlo_mt(z, mu, mtop, _muF_mu_ratio) + smallx_match_z(z, (c2-2*c20)*pow(CF/CA,2)) ) /z;
    else {
      res = qu_nnlo_infmt(z, mu, mtop, _muF_mu_ratio) /z;
    }
  }
  return res;
}
double FO_2_qq(double z, double mu) {
  double res;
  if(!finite_mt) res = C2reg_pointlike_qq(z, _nf, _muF_mu_ratio) /z;
  else {
    double lf = -2.*log(_muF_mu_ratio);
    double c10 = sxc->c10(mu,mtop);
    double c20 = sxc->c20(mu,mtop);
    double c11 = sxc->c11(mu,mtop);
    double c1 = 2*c10;
    double c2 = 2.*c20+c11;
    c2  += (c1+lf)*lf*2.;
    c20 += (c1+lf)*lf/2.;
    if(mu<2*mtop)
      res = ( qq_nnlo_mt(z, mu, mtop, _muF_mu_ratio) + smallx_match_z(z, (c2-2*c20)*pow(CF/CA,2)) ) /z;
    else {
      res = qq_nnlo_infmt(z, mu, mtop, _muF_mu_ratio) /z;
    }
  }
  return res;
}
double FO_2_qqp(double z, double mu) {
  double res;
  if(!finite_mt) res = C2reg_pointlike_qqp(z, _nf, _muF_mu_ratio) /z;
  else {
    double lf = -2.*log(_muF_mu_ratio);
    double c10 = sxc->c10(mu,mtop);
    double c20 = sxc->c20(mu,mtop);
    double c11 = sxc->c11(mu,mtop);
    double c1 = 2*c10;
    double c2 = 2.*c20+c11;
    c2  += (c1+lf)*lf*2.;
    c20 += (c1+lf)*lf/2.;
    if(mu<2*mtop)
      res = ( qqp_nnlo_mt(z, mu, mtop, _muF_mu_ratio) + smallx_match_z(z, (c2-2*c20)*pow(CF/CA,2)) ) /z;
    else {
      res = qqp_nnlo_infmt(z, mu, mtop, _muF_mu_ratio) /z;
    }
  }
  return res;
}




// Fixed-order (nNNLO)
///////////////////////////////////////
dcomplex FO_3_log(dcomplex N) {
  double W1 = (finite_mt ? _W1t : _W1);
  double W2 = (finite_mt ? _W2t : _W2);
  return FO_3_log_PL(N) + W1*FO_2_log_PL(N) + W2*S1(N);
}
double D0D1(double z, double D(int,double)) {
  return 3./2.*D(2,z) - ZETA2*D(0,z);
}
double D0D2(double z, double D(int,double)) {
  return 4./3.*D(3,z) - 2.*ZETA2*D(1,z) + 2.*ZETA3*D(0,z);
}
double D0D3(double z, double D(int,double)) {
  return 5./4.*D(4,z) - 3.*ZETA2*D(2,z) + 6.*ZETA3*D(1,z) - 6.*ZETA4*D(0,z);
}
double FO_3_log(double z, double D(int,double)) {
  double lf = -2.*log(_muF_mu_ratio);
  double W1 = (finite_mt ? _W1t : _W1 );
  double W2 = (finite_mt ? _W2t : _W2 );
  double Clog = FO_3_log_PL(z,D) + W1*FO_2_log_PL(z,D) + W2*FO_1_log_PL(z,D);
  if(__NNLL__) {
    Clog -= 0.953938*D(0,z) *M_PI*M_PI*M_PI;
  }
  Clog += ( 0*lf * D(4,z)
	    + (16.*CA*CA*CA*lf*lf + 0*lf) * D(3,z)
	    + (4.*CA*CA*CA*lf*lf*lf - 18.*CA*CA*b0*lf*lf + 12.*CA*A1*lf) * D(2,z)
	    + (-4.*CA*CA*b0*lf*lf*lf + (8.*CA*A1+4.*CA*b0*b0+4.*CA*CA*W1-32.*CA*CA*CA*ZETA2)*lf*lf + (8*CA*B1-12*CA*b1)*lf) * D(1,z)
	    + ((2.*b0*b0-4.*CA*CA*CA*ZETA2)*lf*lf*lf + (CA*(4.*B1-5.*b1)-3.*CA*b0*W1-2.*A1*b0+6.*CA*CA*b0*ZETA2+32.*CA*CA*CA*ZETA3)*lf*lf
	       + (2.*A2+2.*W1*A1-4.*CA*A1*ZETA2)*lf) * D(0,z) );
  // adding contributions from interference with C2
  Clog += lf*(2.*CA*( 72. * D0D3(z,D)
		      + (2.*_nf - 33.) * D0D2(z,D)
		      + (67.+4*CA*W1 - 90.*ZETA2 - 10./3.*_nf) * D0D1(z,D)
		      + (-101./3. + 33.*ZETA2 + 351./2.*ZETA3 + _nf*(14./9.-2.*ZETA2)) * 2*D(1,z)
		      + (_cd2 + W2 + _cd1*W1 ) * D(0,z)
		      )
	      -2.*b0*( 72. * D(3,z)
		       + (2.*_nf - 33.) * D(2,z)
		       + (67.+4*CA*W1 - 90.*ZETA2 - 10./3.*_nf) * D(1,z)
		       + (-101./3. + 33.*ZETA2 + 351./2.*ZETA3 + _nf*(14./9.-2.*ZETA2)) * D(0,z)
		       ) );
  //double lrf = 2.*log(_muR_muF_ratio);
  //double muRF = _muR_muF_ratio;  _muR_muF_ratio = 1.;
  //Clog += (3*b1*FO_1_log(z,D)+4*b0*FO_2_log(z,D))*lrf + 6*b0*b0*FO_1_log(z,D)*lrf*lrf;
  //_muR_muF_ratio = muRF;
  return Clog;
}
dcomplex FO_3_log_APimprovement(dcomplex N, void *par) {
  double W1 = (finite_mt ? _W1t : _W1);
  double d1 = _cd1 + W1;
  return 1./6.*(pow(APimprovement(S1,N),3) - APimprovement(S1_cube,N))
    + APimprovement(S1,N)*APimprovement(S2,N) - APimprovement(S1_S2,N)
    + d1/2.*(pow(APimprovement(S1,N),2) - APimprovement(S1_sq,N))
    ;
}
double FO_3_log_APimproved(double z, double D(int,double)) {
  double res = APimprovement(z)*FO_3_log(z,D);
  if(APimprExponent) res += InverseMellin(z,FO_3_log_APimprovement,NULL);
  return res;
}
dcomplex FO_3_Nsoftcoll_impr(dcomplex N, void *par) {
  return 2.*CA*log(N)/N*(S2(N)+S1(N)*S1(N)/2.);
}
double FO_3_Nsoftcoll_impr(double z) {
  return InverseMellin(z,FO_3_Nsoftcoll_impr,NULL);
}
double FO_3_delta_const(double mu) {
  double lf = -2.*log(_muF_mu_ratio);
  // not yet known in the full theory => use pointlike value.
  double W1 = _W1;  // (finite_mt ? _W1t : _W1 );
  double W2 = _W2;  // (finite_mt ? _W2t : _W2 );
  double W3 = _W3;  // (finite_mt ? _W3t : _W3 );
  //
  double c = W3 + W2*_cd1 + W1*_cd2 + _cd3;
  //c = (52.1348 - 1.97392*FO_1_delta_const()/M_PI) *M_PI*M_PI*M_PI;  // this corresponds to g0bar3 = 0 only for muF=mH
  if(__NNLL__) c = -FO_3_log(1, mell_const);
  //cout << c/M_PI/M_PI/M_PI << endl;
  //
  // factorization scale dependence
  c += (72.*ZETA3+18.*b0*ZETA2)*lf*lf*lf
    + (-4.*CA*A1*ZETA2-2.*b0*(B1-b1+6.*CA*CA*ZETA3)+b0*b0*(2.*CA*ZETA2+W1)-2.*CA*CA*W1*ZETA2+4.*CA*CA*CA*(ZETA2*ZETA2-6.*ZETA4))*lf*lf
    + (4.*CA*B1*ZETA2-6.*CA*b1*ZETA2-3.*W1*b1+2.*B2-2.*b2+2.*W1*B1+8.*CA*A1*ZETA3)*lf;
  // adding contributions from interference with C2
  c += lf*(2*CA*( 72. * 6*ZETA5
		  + (2.*_nf - 33.) * (-2*ZETA4)
		  + (67.+4*CA*W1 - 90.*ZETA2 - 10./3.*_nf) * ZETA3
		  + (-101./3. + 33.*ZETA2 + 351./2.*ZETA3 + _nf*(14./9.-2.*ZETA2)) * (-ZETA2)
		  )
	   -  2*b0* (_cd2 + W2 + _cd1*W1 ) );
  //double lrf = 2.*log(_muR_muF_ratio);
  //double muRF = _muR_muF_ratio;  _muR_muF_ratio = 1.;
  //c += (2*b2+3*b1*FO_1_delta_const()+4*b0*FO_2_delta_const(mu))*lrf + (7*b0*b1+6*b0*b0*FO_1_delta_const())*lrf*lrf + 4*b0*b0*b0*lrf*lrf*lrf;
  //_muR_muF_ratio = muRF;
  return c;
}
double FO_3_nonlog(double z, double mu) {
  double nonlog = 0.;  // unknown...
  return nonlog;
}
double FO_3_hard(double z, double mu) {
  //double hard = SF(z, mu, FO_3_sx_NLL_RC); // ok checked
  //double lrf = 2.*log(_muR_muF_ratio);
  //hard += FO_1_hard(z,mu) * (3*b1*lrf + 6*b0*b0*lrf*lrf);
  //
  double lf = -2.*log(_muF_mu_ratio);
  double c10 = sxc->c10(mu,mtop);
  double c20 = sxc->c20(mu,mtop);
  double c11 = sxc->c11(mu,mtop);
  double c30 = sxc->c30(mu,mtop);
  double c21 = sxc->c21(mu,mtop);
  if(MSbar) c30 += 8./3.*ZETA3;
  double c1 = 2*c10;
  double c2 = 2.*c20+c11;
  double c3 = 2.*(c30+c21);
  // scale dependence
  c30 += c20*lf + c1*lf*lf/4. + lf*lf*lf/6.;
  c21 += (c2-c20)*lf + 3.*c1*lf*lf/4. + lf*lf*lf/2.;
  c20 += (c1+lf)*lf/2.;
  c2  += (c1+lf)*lf*2.;
  c1  += 2*lf;
  //
  double e01 = CA;
  double e00 = (-11.*CA+2.*_nf*(2.*CF/CA-1.))/12.;
  double e11 = _nf*(26.*CF-23.*CA)/36.;
  double e22 = CA*( (18.*ZETA2-71.)*(CA-2*CF)*_nf + CA*CA*(54*ZETA3+99.*ZETA2-395) ) /108.;
  //
  double hard = (c3*e01*(e01*e01*pow(log(z),2)/2.-3*e01*e00*log(z)+3*e00*e00)
		 - (3*c30+c21)*2*b0*(2*e00*e01-e01*e01*log(z))
		 + 4*c30*b0*b0*e01
		 + 2*c2*(-e01*log(z)+e00)*e11
		 - 4*c20*b0*e11
		 - c1*e22*log(z) )/z;
  //
  return (hard - 12.*MomConsCoeff[2]) * pow(1-z, 2);
}
double FO_3_qg(double z, double mu) {
  return 0;
  //double lf = -2.*log(_muF_mu_ratio);
  double Pgq = CF*(1.+pow(1.-z,2))/z/2.;
  double res = 2*Pgq * pow(log(1-z)-log(z)/2.,5) * (18*CF*CF+100./3.*CF*CA+230./3.*CA*CA)/pow(4.,3);
  return res;
}




//
// Pay attention to the appropriae definition of the soft part..
double UncertaintyCoeff[3] = {0,0,0};
void ComputeUncertaintyCoeff(double mu) {
  UncertaintyCoeff[0] = real(g0expS_order1(2.));
  UncertaintyCoeff[1] = real(g0expS_order2(2.));
  UncertaintyCoeff[2] = real(g0expS_order3(2.));
}









double FO_log(double z, double D(int,double)) {
  double res = 0;
  if(order == 1) res = FO_1_log(z,D);
  if(order == 2) res = FO_2_log(z,D);
  if(order == 3) res = FO_3_log(z,D);
  return res;
}
double FO_log_APimproved(double z, double D(int,double)) {
  double res = 0;
  if(order == 1) res = FO_1_log_APimproved(z,D);
  if(order == 2) res = FO_2_log_APimproved(z,D);
  if(order == 3) res = FO_3_log_APimproved(z,D);
  return res;
}
double FO_Nsoftcoll_impr(double z) {
  double res = 0;
  if(order == 1) res = FO_1_Nsoftcoll_impr(z);
  if(order == 2) res = FO_2_Nsoftcoll_impr(z);
  if(order == 3) res = FO_3_Nsoftcoll_impr(z);
  return res;
}
double FO_nonlog(double z, double mu) {
  double res = 0;
  if(order == 1) res = FO_1_nonlog(z,mu);
  if(order == 2) res = FO_2_nonlog(z,mu);
  if(order == 3) res = FO_3_nonlog(z,mu);
  return res;
}
double FO_hard(double z, double mu) {
  if(!finite_mt) return 0;
  double res = 0;
  if(order == 1) res = FO_1_hard(z,mu);
  if(order == 2) res = FO_2_hard(z,mu);
  if(order == 3) res = FO_3_hard(z,mu);
  return res;
}
double FO_delta_const(double mu) {
  double res = 0;
  if(order == 1) res = FO_1_delta_const();
  if(order == 2) res = FO_2_delta_const(mu);
  if(order == 3) res = FO_3_delta_const(mu);
  return res;
}
double FO_qg(double z, double mu) {
  double res = 0;
  if(order == 1) res = FO_1_qg(z, mu);
  if(order == 2) res = FO_2_qg(z, mu);
  if(order == 3) res = FO_3_qg(z, mu);
  return res;
}
double FO_qqb(double z, double mu) {
  double res = 0;
  if(order == 1) res = FO_1_qqb(z, mu);
  if(order == 2) res = FO_2_qqb(z, mu);
  if(order == 3) res = 0; // not yet implemented
  return res;
}
double FO_qq(double z, double mu) {
  double res = 0;
  if(order == 2) res = FO_2_qq(z, mu);
  if(order == 3) res = 0; // not yet implemented
  return res;
}
double FO_qqp(double z, double mu) {
  double res = 0;
  if(order == 2) res = FO_2_qqp(z, mu);
  if(order == 3) res = 0; // not yet implemented
  return res;
}








/////////////////////////////
//  LUMINOSITY
double lumi_gg(double x1, double x2, double mu) {
  //if(x1<1.e-7) x1=1.e-7;
  //if(x2<1.e-7) x2=1.e-7;
  return LHAPDF::xfx(x1,mu,0) * LHAPDF::xfx(x2,mu,0) /x1/x2;
}
double lumi_qg(double x1, double x2, double mu) {
  double xfx1[13], xfx2[13];
  LHAPDF::xfx(x1, mu, xfx1);
  LHAPDF::xfx(x2, mu, xfx2);
  double res = 0;
  for(int i=1; i<=_nf; i++) {
    res += (xfx1[6+i] + xfx1[6-i]) * xfx2[6+0];
    res += (xfx2[6+i] + xfx2[6-i]) * xfx1[6+0];
  }
  return res/x1/x2;
}
double lumi_qqb(double x1, double x2, double mu) {
  double xfx1[13], xfx2[13];
  LHAPDF::xfx(x1, mu, xfx1);
  LHAPDF::xfx(x2, mu, xfx2);
  double res = 0;
  for(int i=-_nf; i<=_nf; i++) {
    if(i==0) continue;
    res += xfx1[6+i] * xfx2[6-i];
  }
  return res/x1/x2;
}
double lumi_qq(double x1, double x2, double mu) {
  double xfx1[13], xfx2[13];
  LHAPDF::xfx(x1, mu, xfx1);
  LHAPDF::xfx(x2, mu, xfx2);
  double res = 0;
  for(int i=-_nf; i<=_nf; i++) {
    if(i==0) continue;
    res += xfx1[6+i] * xfx2[6+i];
  }
  return res/x1/x2;
}
double lumi_qqp(double x1, double x2, double mu) {
  double xfx1[13], xfx2[13];
  LHAPDF::xfx(x1, mu, xfx1);
  LHAPDF::xfx(x2, mu, xfx2);
  double res = 0;
  for(int i=-_nf; i<=_nf; i++) {
    for(int j=-_nf; j<=_nf; j++) {
      if(i!=j && i!=-j && i!=0 && j!=0) res += xfx1[6+i] * xfx2[6+j];
    }
  }
  return res/x1/x2;
}
double lumi_SS(double x1, double x2, double mu) {
  double xfx1[13], xfx2[13];
  LHAPDF::xfx(x1, mu, xfx1);
  LHAPDF::xfx(x2, mu, xfx2);
  double S1=0, S2=0;
  for(int i=1; i<=_nf; i++) {
    S1 += xfx1[6+i]+xfx1[6-i];
    S2 += xfx2[6+i]+xfx2[6-i];
  }
  return S1*S2/x1/x2;
}
// this is = Vu^2 + Vd^2 + Vs^2 + ...
double lumi_VV(double x1, double x2, double mu) {
  double xfx1[13], xfx2[13];
  LHAPDF::xfx(x1, mu, xfx1);
  LHAPDF::xfx(x2, mu, xfx2);
  double VV=0, V1=0, V2=0;
  for(int i=1; i<=_nf; i++) {
    V1 = xfx1[6+i]-xfx1[6-i];
    V2 = xfx2[6+i]-xfx2[6-i];
    VV += V1*V2;
  }
  return VV/x1/x2;
}
/*
// this is = V^2/nf + 1/2 [ V3^2 + V8^2/3 + V15^2/6 + V24^2/10 (+ V35^2/15) ]
double lumi_VV(double x1, double x2, double mu) {
  double xfx1[13], xfx2[13];
  LHAPDF::xfx(x1, mu, xfx1);
  LHAPDF::xfx(x2, mu, xfx2);
  double Vi1=0, Vi2=0, VV=0, V1=0, V2=0;
  for(int i=1; i<=_nf; i++) {
    V1 += xfx1[6+i]-xfx1[6-i];
    V2 += xfx2[6+i]-xfx2[6-i];
  }
  for(int i=2; i<=_nf; i++) {
    Vi1=0; Vi2=0;
    for(int j=1; j<=i-1; j++) {
      Vi1 += xfx1[6+j]-xfx1[6-j];
      Vi2 += xfx2[6+j]-xfx2[6-j];
    }
    Vi1 -= (i-1)*(xfx1[6+i]-xfx1[6-i]);
    Vi2 -= (i-1)*(xfx2[6+i]-xfx2[6-i]);
    VV += Vi1*Vi2/(i-1.)/double(i);
  }
  return (VV + V1*V2/double(_nf))/x1/x2;
}
*/
// this is = 1/2 [ T3^2 + T8^2/3 + T15^2/6 + T24^2/10 (+ T35^2/15) ]
double lumi_TT(double x1, double x2, double mu) {
  double xfx1[13], xfx2[13];
  LHAPDF::xfx(x1, mu, xfx1);
  LHAPDF::xfx(x2, mu, xfx2);
  double T1=0, T2=0, TT=0;
  for(int i=2; i<=_nf; i++) {
    T1=0; T2=0;
    for(int j=1; j<=i-1; j++) {
      T1 += xfx1[6+j]+xfx1[6-j];
      T2 += xfx2[6+j]+xfx2[6-j];
    }
    T1 -= (i-1)*(xfx1[6+i]+xfx1[6-i]);
    T2 -= (i-1)*(xfx2[6+i]+xfx2[6-i]);
    TT += T1*T2/(i-1.)/double(i);
  }
  return TT/x1/x2;
}
/////////////////







////////////////////////////////////////
// integrating delta term
struct int1dp {
  double tau, mu;
};
int LumChannel=0;
double Lumij_integrand(double x, void *par) {
  int1dp p = *(int1dp *) par;
  double tau = p.tau;
  double mu  = p.mu;
  double res = 0;
  if(LumChannel==0) res = lumi_gg (tau/x,x,mu);
  if(LumChannel==1) res = lumi_qg (tau/x,x,mu);
  if(LumChannel==2) res = lumi_SS (tau/x,x,mu); // Singlet-Singlet
  if(LumChannel==3) res = lumi_TT (tau/x,x,mu);
  if(LumChannel==4) res = lumi_VV (tau/x,x,mu);
  return res/x;
}
double Lumij(double tau, double mu) {
  int1dp p = { tau, mu };
  return gauss(Lumij_integrand, tau, 1., INTEGRAL_PRECISION, NULL, &p);
}
double Lum(double tau, double mu) {
  LumChannel = 0;
  return Lumij(tau,mu);
}
double ChebLum(double z) {
  return Lumij(z, muM*_muF_mu_ratio);
  //return Lumij(z, muF);
}
double ChebRecLum(double *c, int N, double z, double beta, int gamma) {
  double value = 0;
  for(int i=0; i<=N; i++) {
    for(int j=0; j<=gamma; j++) {
      value += c[i] * pow(z,j-beta)* pow(-1.,j) * gsl_sf_choose(gamma,j) *pow(-log(z),i)/gsl_sf_fact(i);
    }
  }
  return value;
}
////////////////////////////////////////




double distr(double z, double mu) {
  double res = 0;
  if(__exact__)
    res = FO_log(z,Dk);
  else if(__soft_part__) {
    if(__log_type__ == 2)
      res = FO_log(z,DMPk);
    else
      res = FO_log(z,Dk);
  }
  //
  //double lrf = 2.*log(_muR_muF_ratio);
  //if(order == 2) res += 3*b0*FO_1_log(z,Dk)*lrf;
  //if(order == 3) res += (3*b1+6*b0*b0*lrf)*lrf*FO_1_log(z,Dk) + 4*b0*FO_2_log(z,Dk)*lrf;
  return res;
}
double reg(double z, double mu) {
  double res = 0;
  if(__exact__) {
    res = FO_nonlog(z,mu);
  }
  else {
    if(__soft_part__ && __log_type__==1 && APtype!=0)
      res = FO_log_APimproved(z,Dtk) - FO_log(z,Dk);
    if(__soft_part__ && __log_type__==2 && __Nsoftcoll__)
      res = FO_Nsoftcoll_impr(z);
    if(__hard_part__)
      res += FO_hard(z,mu);
  }
  /*
  if(__exact__ || __soft_part__) {
    double lrf = 2.*log(_muR_muF_ratio);
    if(order == 2) res += 3*b0*FO_1_nonlog(z,mu)*lrf;
    if(order == 3) res += (3*b1+6*b0*b0*lrf)*lrf*FO_1_nonlog(z,mu) + 4*b0*FO_2_nonlog(z,mu)*lrf;
  }
  */
  return res;
}
double delta(double mu) {
  double res = 0;
  if(__exact__ || __soft_part__) {
    res = FO_delta_const(mu);
    if(__log_type__ == 2) res += FO_log(0, mell_const);
    /*
    double lrf = 2.*log(_muR_muF_ratio);
    if(order == 1) res += 2*b0*lrf;
    if(order == 2) res += (2*b1+3*b0*FO_1_delta_const())*lrf + 3*b0*b0*lrf*lrf;
    if(order == 3) res += (2*b2+3*b1*FO_1_delta_const()+4*b0*FO_2_delta_const(mu))*lrf
		     + (7*b0*b1+6*b0*b0*FO_1_delta_const())*lrf*lrf + 4*b0*b0*b0*lrf*lrf*lrf;
    */
  }
  return res;
}







// integrand of regular part
double reg_int(double z, double x, double tau, double mu) {
  return lumi_gg(tau/z/x, x, mu*_muF_mu_ratio) * reg(z, mu) /z /x;
}
double reg_int_chvar(double t1, double t2, double tau, double mu) {
  double z = 1.-(1.-tau)*t1;
  double x = 1.-(1.-tau/z)*t2;
  return reg_int(z, x, tau, mu) * (1-tau) * (1-tau/z);
}



// integrand of distributional part
double dlum_int(double z, double x, double tau, double mu) {
  return lumi_gg(tau/z/x, x, mu*_muF_mu_ratio) /z /x;
}
double dlum_int_chvar(double z, double t2, double tau, double mu) {
  double x = 1.-(1.-tau/z)*t2;
  return dlum_int(z, x, tau, mu) * (1-tau/z);
}
double distr_int(double z, double t2, double tau, double mu) {
  return ( dlum_int_chvar(z, t2, tau, mu) - dlum_int_chvar(1, t2, tau, mu) ) * distr(z, mu);
}
double distr_int_chvar(double t1, double t2, double tau, double mu) {
  double z = 1.-(1.-tau)*t1;
  return distr_int(z, t2, tau, mu) * (1-tau);
}

// integrand of the residual distributional part
double distr_residual_integrand(double z, void *par) {
  double mu  = *(double*)par;
  double res = distr(z,mu);
  return res;
}
double distr_residual(double tau, double mu) {
  double res = gauss(distr_residual_integrand, 0., tau, INTEGRAL_PRECISION, NULL, &mu);
  //cout << "!!! >>>>>" << res << endl;
  return res;
}





// non-gg channels contributions
double qg_int(double z, double x, double tau, double mu) {
  return lumi_qg(tau/z/x, x, mu*_muF_mu_ratio) * FO_qg(z, mu) /z /x;
}
double qg_int_chvar(double t1, double t2, double tau, double mu) {
  double z = 1.-(1.-tau)*t1;
  double x = 1.-(1.-tau/z)*t2;
  return qg_int(z, x, tau, mu) * (1-tau) * (1-tau/z);
}
//
double qqb_int(double z, double x, double tau, double mu) {
  return lumi_qqb(tau/z/x, x, mu*_muF_mu_ratio) * FO_qqb(z, mu) /z /x;
}
double qqb_int_chvar(double t1, double t2, double tau, double mu) {
  double z = 1.-(1.-tau)*t1;
  double x = 1.-(1.-tau/z)*t2;
  return qqb_int(z, x, tau, mu) * (1-tau) * (1-tau/z);
}
//
double qq_int(double z, double x, double tau, double mu) {
  return lumi_qq(tau/z/x, x, mu*_muF_mu_ratio) * FO_qq(z, mu) /z /x;
}
double qq_int_chvar(double t1, double t2, double tau, double mu) {
  double z = 1.-(1.-tau)*t1;
  double x = 1.-(1.-tau/z)*t2;
  return qq_int(z, x, tau, mu) * (1-tau) * (1-tau/z);
}
//
double qqp_int(double z, double x, double tau, double mu) {
  return lumi_qqp(tau/z/x, x, mu*_muF_mu_ratio) * FO_qqp(z, mu) /z /x;
}
double qqp_int_chvar(double t1, double t2, double tau, double mu) {
  double z = 1.-(1.-tau)*t1;
  double x = 1.-(1.-tau/z)*t2;
  return qqp_int(z, x, tau, mu) * (1-tau) * (1-tau/z);
}





bool __all_channels__ = false;
int integrandCUBA(const int *ndim, const double x[], const int *ncomp, double res[], void *pars) {
  int1dp p = *(int1dp *) pars;
  double tau = p.tau;
  double mu  = p.mu;
  res[0] = 0;
  res[0] += reg_int_chvar(x[0],x[1],tau,mu);
  if(__exact__ || __soft_part__)
    res[0] += distr_int_chvar(x[0],x[1],tau,mu);
  if(__all_channels__ && __exact__) {
    res[0] += qg_int_chvar(x[0],x[1],tau,mu);
    res[0] += qqb_int_chvar(x[0],x[1],tau,mu);
    if(order > 1)
      res[0] += qq_int_chvar(x[0],x[1],tau,mu) + qqp_int_chvar(x[0],x[1],tau,mu);
  }
  return 0;
}
double integral2d(double tau, double mu, double *err = NULL) {
  double res = 0;
  int1dp p = { tau, mu };
  //
  double cres[1], error[1], prob[1];
  double epsrel=INTEGRAL_PRECISION, epsabs=1.e-10;
  int last = 4;
  int verbose = 0;
  int nregions, neval, fail;
  //system("export CUBACORES=8");
  Cuhre(2, 1, integrandCUBA, &p, 1,
	epsrel, epsabs, verbose | last,
	0, 500000, 9,
	NULL, NULL,
	&nregions, &neval, &fail, cres, error, prob);
  // Suave(2, 1, integrandCUBA, &p,
  // 	epsrel, epsabs, verbose | last, 0,
  // 	0, 500000, 1000, 25.,
  // 	&nregions, &neval, &fail, cres, error, prob);
  // Vegas(2, 1, integrandCUBA, &p,
  // 	epsrel, epsabs, verbose | last, 0,
  // 	0, 500000, 1000, 500, 1000,
  // 	0, NULL,
  // 	&neval, &fail, cres, error, prob);
  //  
  if(fail == -1) {
    cout << "  ERROR: dimension out of range." << endl;
  } else if(fail>0) {
    cout << "  ERROR: precision not reached." << endl;
  }
  double lum_tau = Lum(tau, mu*_muF_mu_ratio);
  res += cres[0]/lum_tau;
  if(err != NULL) *err = error[0]/lum_tau;
  return res;
}


double xs(double M, double sqrts, double *err = NULL) {
  double tau = M*M/sqrts/sqrts;
  double res = integral2d(tau, M, err);
  res += delta(M);
  res -= distr_residual(tau, M);
  return res;
}




dcomplex brn_q_ep0(dcomplex x) {
  dcomplex logx = log(x+I*1e-60);
  dcomplex HPL2 = 0.5*logx*logx;
  return -16.*(1.+x)*(1.+x)/x*HPL2 + 32.*(x-1.)*(x-1.)/x;
}
dcomplex sborn_mass(double mH) {
  int number_of_quarks = 1;
  vector<double> m;
  m.push_back(mtop);
  if(mbot>0) { number_of_quarks++; m.push_back(mbot); }
  if(mcha>0) { number_of_quarks++; m.push_back(mcha); }
  dcomplex res = 0;
  for(int i=0; i<number_of_quarks; i++) {
    dcomplex Wq = 4.*m[i]*m[i]/mH/mH;
    dcomplex Xq = -Wq/pow(1.+sqrt(1.-Wq),2);
    res += brn_q_ep0(Xq)*Wq*Wq /64.*(-3./4.);
  }
  return res/M_PI;
}
double xs_LO_prefactor(double mH) {
  double pref = 35.0309; // = Gf*pi/sqrt(2)/288 with the Gf in pb
  dcomplex born = sborn_mass(mH);
  return pref*real(born*conj(born));
}











void compute(double mH, double sqrts, double LO, double NLO, double NNLO,
	     double *rexact, double *rsoft2, double *rsoft1, double *rNsoft,
	     double *rhighenergy, double *momconsunc,
	     double *rsoft0, double *rsoftAP, double *rNsoftcoll, double *rNNLL, double *rNsoftdFMMV) {
  double as = alphas(mH*_muF_mu_ratio*_muR_muF_ratio);
  //cout << "alpha_s = "<< as << endl;
  double api = as/M_PI;
  double fact = pow(api,order) * LO;
  double tau = mH*mH/sqrts/sqrts;
  SetScales(mH);
  LumChannel = 0;  ApproxLuminosity(ChebCoeff_gg, tau, Nchebyshev, MPbeta, MPgamma);
  LumChannel = 1;  ApproxLuminosity(ChebCoeff_qg, tau, Nchebyshev, MPbeta, MPgamma);
  LumChannel = 2;  ApproxLuminosity(ChebCoeff_SS, tau, Nchebyshev, MPbeta, MPgamma);
  LumChannel = 3;  ApproxLuminosity(ChebCoeff_TT, tau, Nchebyshev, MPbeta, MPgamma);
  LumChannel = 4;  ApproxLuminosity(ChebCoeff_VV, tau, Nchebyshev, MPbeta, MPgamma);
  /*
  LumChannel = 4;
  for(int i=0; i<20; i++) {
    double z = 0.999*exp(i/(20-1.)*log(tau));
    double l1 = pow(z,MPbeta)/pow(1-z,MPgamma)*ChebLum(z);
    double l2 = pow(z,MPbeta)/pow(1-z,MPgamma)*ChebRecLum(ChebCoeff_VV, Nchebyshev, z, MPbeta, MPgamma);
    cout << setw(12) << z
	 << setw(15) << l1
	 << setw(15) << l2
	 << setw(15) << (l2-l1)/l1
	 << endl;
  }
  abort();
  //*/
  //
  string channel = (__all_channels__ ? "all channels:" : "gg channel:");
  if(!quiet) cout << channel << endl;
  string sassig[4] = { "  sig0 = ",
		       "  as sig1 = ",
		       "  as^2 sig2 = ",
		       "  as^3 sig3 = " };
  double err;
  double lrf = 2.*log(_muR_muF_ratio);
  double mur_dep_term = 0; // using terms of previous orders *at* mur
  NNLO -= NLO;
  NLO -= LO;
  if(exactmuRdep) {
    if(order == 1) mur_dep_term = api*LO*2*b0*lrf;
    if(order == 2) mur_dep_term = api*(api*2*b1*LO + 3*b0*NLO) *lrf - api*api*3*b0*b0*LO*lrf*lrf;
    if(order == 3) mur_dep_term = api*(api*api*2*b2*LO + api*3*b1*NLO + 4*b0*NNLO) *lrf
		     - api*api*(api*7*b0*b1*LO + 6*b0*b0*NLO) *lrf*lrf + api*api*api*4*b0*b0*b0*LO*lrf*lrf*lrf;
  }
  //cout << mur_dep_term << endl;
  //
  __exact__ = true; // compute the full gg contribution to the hadronic xs
  __log_type__ = 0;
  APtype = 0;
  if(rexact != NULL && order < 3) {
    //double mufrat = _muF_mu_ratio;
    //if(order==2) _muF_mu_ratio = 1;
    *rexact = xs(mH, sqrts, &err) * fact + mur_dep_term;
    //_muF_mu_ratio = mufrat;
    //if(order==2)*rexact += MellinInversion(tau, &err) * fact;
    err *= fact;
    if(!quiet) cout << sassig[order] << *rexact/LO << " +/- " << err/LO
		    << "  (exact)" << endl;
  }
  //
  __exact__ = false; // compute only approximations
  __soft_part__ = true; // turn on soft
  __hard_part__ = false; // leave off hard
  __log_type__ = 0;
  if(rsoft0 != NULL) {
    *rsoft0 = xs(mH, sqrts, &err) * fact + mur_dep_term;
    //if(APimprExponent)
    //  *rsoft0 = MellinInversion(tau, &err) * fact + mur_dep_term;
    //else
    //  *rsoft0 = xs(mH, sqrts, &err) * fact + mur_dep_term;
    err *= fact;
    if(!quiet) cout << sassig[order] << *rsoft0/LO << " +/- " << err/LO
		    << "  (soft-0 only)" << endl;
  }
  //
  __log_type__ = 1;
  APtype = 2;
  if(rsoft2 != NULL) {
    if(APimprExponent)
      *rsoft2 = MellinInversion(tau, &err) * fact + mur_dep_term;
    else
      *rsoft2 = xs(mH, sqrts, &err) * fact + mur_dep_term;
    err *= fact;
    if(!quiet) cout << sassig[order] << *rsoft2/LO << " +/- " << err/LO
		    << "  (soft_2 only)" << endl;
  }
  //
  __log_type__ = 1;
  APtype = 1;
  if(rsoft1 != NULL) {
    if(APimprExponent)
      *rsoft1 = MellinInversion(tau, &err) * fact + mur_dep_term;
    else
      *rsoft1 = xs(mH, sqrts, &err) * fact + mur_dep_term;
    err *= fact;
    if(!quiet) cout << sassig[order] << *rsoft1/LO << " +/- " << err/LO
		    << "  (soft_1 only)" << endl;
  }
  //
  __log_type__ = 1;
  APtype = -1;
  if(rsoftAP != NULL) {
    if(APimprExponent)
      *rsoftAP = MellinInversion(tau, &err) * fact + mur_dep_term;
    else
      *rsoftAP = xs(mH, sqrts, &err) * fact + mur_dep_term;
    err *= fact;
    if(!quiet) cout << sassig[order] << *rsoftAP/LO << " +/- " << err/LO
		    << "  (soft_inf (full AP) only)" << endl;
  }
  //
  APtype = 0;
  if(rNsoft != NULL) {
    __log_type__ = 3;
    *rNsoft = MellinInversion(tau, &err) * fact + mur_dep_term;
    //__log_type__ = 2;
    //*rNsoft = xs(mH, sqrts, &err) * fact + mur_dep_term;
    err *= fact;
    if(!quiet) cout << sassig[order] << *rNsoft/LO << " +/- " << err/LO
		    << "  (N-soft only)" << endl;
  }
  //
  if(rNsoftdFMMV != NULL) {
    dFMMV = true;
    __log_type__ = 3;
    *rNsoftdFMMV = MellinInversion(tau, &err) * fact + mur_dep_term;
    err *= fact;
    if(!quiet) cout << sassig[order] << *rNsoftdFMMV/LO << " +/- " << err/LO
		    << "  (N-soft only)" << endl;
    dFMMV = false;
  }
  //
  __log_type__ = 2;
  if(rNsoftcoll != NULL) {
    __Nsoftcoll__ = true;
    *rNsoftcoll = xs(mH, sqrts, &err) * fact + mur_dep_term;
    err *= fact;
    if(!quiet) cout << sassig[order] << *rNsoftcoll/LO << " +/- " << err/LO
		    << "  (N-soft-collinear only)" << endl;
    __Nsoftcoll__ = false;
  }
  //
  if(rNNLL != NULL) {
    __NNLL__ = true;
    *rNNLL = xs(mH, sqrts, &err) * fact + mur_dep_term;
    err *= fact;
    if(!quiet) cout << sassig[order] << *rNNLL/LO << " +/- " << err/LO
		    << "  (N-soft-collinear only)" << endl;
    __NNLL__ = false;
  }
  //
  __soft_part__ = false; // turn soft off
  __hard_part__ = true;  // turn hard on
  if(rhighenergy != NULL) {
    // momentum conservation
    ComputeMomConsCoeff(mH);
    *rhighenergy = xs(mH, sqrts, &err) * fact;
    err *= fact;
    if(!quiet) cout << sassig[order] << *rhighenergy/LO << " +/- " << err/LO
		    << "  (high-energy --- must be added to a soft)" << endl;
  }
  //
  if(momconsunc != NULL) {
    if(rhighenergy != NULL) {
      *momconsunc = *rhighenergy;
    } else {
      ComputeMomConsCoeff(mH);
      *momconsunc = xs(mH, sqrts, &err) * fact;
    }
    double MCcoeff = MomConsCoeff[order-1];
    ResetMomConsCoeff();
    *momconsunc -= xs(mH, sqrts, &err) * fact; // mom-cons function
    //
    APtype = 2;  __log_type__ = 1;
    ComputeUncertaintyCoeff(mH);
    //cout << UncertaintyCoeff[order-1]/MCcoeff << endl;
    //
    *momconsunc *= -UncertaintyCoeff[order-1]/MCcoeff;
    //cout << *momconsunc << endl;
    err *= fact*UncertaintyCoeff[order-1]/MCcoeff;
    if(!quiet) cout << sassig[order] << *momconsunc*0.05/LO << " +/- " << err*0.05/LO
		    << "  (momcons uncertainty)" << endl;
  }
}














// interface to LHAPDF
int pdf_init(string SET_NAME, int SUBSET) {
#ifndef LHAPDF_MAJOR_VERSION
  LHAPDF::setVerbosity(LHAPDF::SILENT);
#endif
  LHAPDF::initPDFSet(SET_NAME, SUBSET);
  cout << ">>> Using PDFs:  " << SET_NAME << " ,  subset: " << SUBSET
       << " ,  with alpha_s(mZ) = " << LHAPDF::alphasPDF(mZ) << endl;
  if(!quiet) cout << "sigma = sigma0 * [ 1 + as sig1 + as^2 sig2 + as^3 sig3 + ...]" << endl;
  return 0;
}
//




